// FirmwareData.cpp: implementation of the CFirmwareData class.
//
//////////////////////////////////////////////////////////////////////
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FirmwareData.cpp.
 *
 * The Initial Developer of the Original Code is
 * Fenestra Software LLC.
 * Portions created by the Initial Developer are Copyright (C) 2002 Fenestra Software LLC
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "stdafx.h"
#include "Phantom.h"
#include "FirmwareData.h"
#include "PhantomCtrl.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFirmwareData::CFirmwareData()
{
	Image = new BYTE[IMAGE_MAXLEN];
	memset(Image,0,IMAGE_MAXLEN);
	memset(m_bData, 0, SREC_DATA_SIZE);
	iStart = 0;
	m_bType=0;
	m_iAddr=0;
	m_bType=0;
	m_error = SREC_OK;
}

CFirmwareData::~CFirmwareData()
{
	delete[] Image;
}


// Function name	: CFirmwareData::Load
// Description	    : Opens and memory maps the firmware file.
//					: Then reads and decodes it one line at a time.
// Return type		: BOOL 
// Argument         : LPSTR FileName
// Argument         : CPhantomCtrl *parent
BOOL CFirmwareData::Load(LPSTR FileName,CPhantomCtrl *parent)
{
	CFirmwareFile FirmFile;
	char buf[256];
	int line = 0;
	//Open the firmware file
	if(FirmFile.Open(FileName)==false){
		parent->ReportError(2016, GetLastError());
		return false;
	}

	// For each line in the file
	while (FirmFile.gets(buf, sizeof(buf))!=NULL) {
		line++;		
		int error;
		// Decode it into the buffer.
		error = Decode(buf);
		if(error != SREC_OK){
			parent->ReportError(2016, strError());
			FirmFile.Close();
			return false;
		}
	}
	//Close the firmware file.
	FirmFile.Close();
	return true;
}

// Function name	: CFirmwareData::GetChecksum
// Description	    : Checksums the buffer and returns the value of the checksum
// Return type		: int 
int CFirmwareData::GetChecksum()
{
	// Compute image checksum
	int cksum = 0;
	int cksumlen = (length < 0x4c00) ? length : 0x4c00;
    for (int i = 0; i < cksumlen; i++){
		cksum += Image[i];
	}
	cksum = cksum &0xffff;
	return cksum;
}

//*
//*  srec.c
//*
//*  S-record routines.
//*
//*  The contents of this file are subject to the Mozilla Public License
//*  Version 1.0 (the "License"); you may not use this file except in
//*  compliance with the License. You may obtain a copy of the License at
//*  http:www.mozilla.org/MPL/
//*
//*  Software distributed under the License is distributed on an "AS IS"
//*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
//*  License for the specific language governing rights and limitations
//*  under the License.
//*
//*  The Original Code is Firmdl code, released October 3, 1998.
//*
//*  The Initial Developer of the Original Code is Kekoa Proudfoot.
//*  Portions created by Kekoa Proudfoot are Copyright (C) 1998, 1999
//*  Kekoa Proudfoot. All Rights Reserved.
//*
//*  Contributor(s): Kekoa Proudfoot <kekoa@graphics.stanford.edu>
//*

// Function name	: CFirmwareData::Decode
// Description	    : Decodes the firmware file into a buffer that can be uploaded to the RCX
//					: one line at a time.
// Return type		: int 
// Argument         : char *LineIn
int CFirmwareData::Decode(char *LineIn)
{
    int len, pos = 0, count, alen, sum = 0;
    BYTE *line = (BYTE *)LineIn;

    if (!line){
		m_error = SREC_NULL;
		return m_error;
	}

    for (len = 0; line[len]; len++)
		if (line[len] == '\n' || line[len] == '\r')
			break;

    if (len < 4){
		m_error = SREC_INVALID_HDR;
		return m_error;
	}

    if (line[0] != 'S'){
		m_error = SREC_INVALID_HDR;
		return m_error;
	}

    for (pos = 1; pos < len; pos++) {
		if (C1(line, pos) < 0){
			m_error = SREC_INVALID_CHAR;
			return m_error;
		}
    }

    m_bType = C1(line, 1);
    count = C2(line, 2);

    if (m_bType > 9){
		m_error = SREC_INVALID_TYPE;
		return m_error;
	}
    alen = ltab[m_bType];
    if (alen == 0){
		m_error = SREC_INVALID_TYPE;
		return m_error;
	}
    if (len < alen + 6){
		m_error = SREC_TOO_SHORT;
		return m_error;
	}
    if (count > alen + SREC_DATA_SIZE + 2){
		m_error = SREC_TOO_LONG;
		return m_error;
	}
    if (len != count * 2 + 4){
		m_error = SREC_INVALID_LEN;
		return m_error;
	}

    sum += count;

    len -= 4;
    line += 4;

    m_iAddr = 0;
    for (pos = 0; pos < alen; pos += 2) {
		BYTE value = C2(line, pos);
		m_iAddr = (m_iAddr << 8) | value;
		sum += value;
	}

    len -= alen;
    line += alen;

    for (pos = 0; pos < len - 2; pos += 2) {
		BYTE value = C2(line, pos);
		m_bData[pos / 2] = value;
		sum += value;
	}

    m_iCount = count - (alen / 2) - 1;

    sum += C2(line, pos);

    if ((sum & 0xff) != 0xff){
		m_error = SREC_INVALID_CKSUM;
		return m_error;
	}



	switch(m_bType){
	case 0:
		break;
	case 1:
		if (m_iAddr < IMAGE_START || m_iAddr + m_iCount > IMAGE_START + IMAGE_MAXLEN) {
			m_error = SREC_INVALID_ADDRESS;
			return m_error;
		}
		if (m_iAddr + m_iCount - IMAGE_START > length)
			length = m_iAddr + m_iCount - IMAGE_START;

		memcpy(&Image[m_iAddr - IMAGE_START], &m_bData, m_iCount);

		break;
	case 9:
		if (m_iAddr < IMAGE_START || m_iAddr > IMAGE_START + IMAGE_MAXLEN) {
			m_error = SREC_INVALID_ADDRESS;
			return m_error;
		}
		iStart = m_iAddr;
		break;
	default:
		m_error = SREC_INVALID_TYPE;
		return m_error;
	}

	m_error = SREC_OK;
    return SREC_OK;
}


// Function name	: CFirmwareData::strError
// Description	    : Returns a printable string for the current value of m_error
// Return type		: LPSTR 
LPSTR CFirmwareData::strError()
{
    switch (m_error) {
    case SREC_OK: return "No Error";
    case SREC_NULL: return "Null String Error";
    case SREC_INVALID_HDR: return "Invalid Header";
    case SREC_INVALID_CHAR: return "Invalid Character";
	case SREC_INVALID_TYPE: return "Invalid Type";
    case SREC_TOO_SHORT: return "Line Too Short";
    case SREC_TOO_LONG: return "Line Too Long";
    case SREC_INVALID_LEN: return "Length Error";
    case SREC_INVALID_CKSUM: return "Checksum Error";
    case SREC_INVALID_ADDRESS: return "Address Error";
    default: return "Unknown Error";
    }
}


