// FirmwareFile.cpp: implementation of the CFirmwareFile class.
//
//////////////////////////////////////////////////////////////////////
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FirmwareFile.cpp.
 *
 * The Initial Developer of the Original Code is
 * Fenestra Software LLC.
 * Portions created by the Initial Developer are Copyright (C) 2002 Fenestra Software LLC
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "stdafx.h"
#include "FirmwareFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFirmwareFile::CFirmwareFile()
{
	m_iCurrPos = 0;
	m_pFileData = NULL;
	m_MappingHandle = INVALID_HANDLE_VALUE;
	m_FileHandle = INVALID_HANDLE_VALUE;

}

CFirmwareFile::~CFirmwareFile()
{
	if(m_pFileData)
		UnmapViewOfFile(m_pFileData);
	if(m_MappingHandle != INVALID_HANDLE_VALUE)
		CloseHandle(m_MappingHandle);
	if(m_FileHandle != INVALID_HANDLE_VALUE)
		CloseHandle(m_FileHandle);
}


// Function name	: CFirmwareFile::Open
// Description	    : Open and Map the file.
// Return type		: BOOL 
// Argument         : LPSTR FileName
BOOL CFirmwareFile::Open(LPSTR FileName)
{
	m_FileHandle = CreateFile(FileName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(m_FileHandle == INVALID_HANDLE_VALUE){
			return false;
	}


	GetFileInformationByHandle(m_FileHandle, &m_FileInfo);


	m_MappingHandle = CreateFileMapping(m_FileHandle,NULL,PAGE_READONLY,0,0,"FirmwareFile");
	if(m_MappingHandle == INVALID_HANDLE_VALUE){
		CloseHandle(m_FileHandle);
		return false;
	}


	m_pFileData = (LPSTR)MapViewOfFile(m_MappingHandle,FILE_MAP_READ, 0,0,0);
	if(!m_pFileData){
		CloseHandle(m_MappingHandle);
		CloseHandle(m_FileHandle);
		return false;
	}
	m_iCurrPos = 0;
	return true;
}


// Function name	: CFirmwareFile::Close
// Description	    : Unmap and close the file.
// Return type		: void 
void CFirmwareFile::Close()
{
	UnmapViewOfFile(m_pFileData);
	CloseHandle(m_MappingHandle);
	CloseHandle(m_FileHandle);
	m_pFileData = NULL;
	m_MappingHandle = INVALID_HANDLE_VALUE;
	m_FileHandle = INVALID_HANDLE_VALUE;
}


// Function name	: CFirmwareFile::gets
// Description	    : Get String from file.
// Return type		: LPSTR 
// Argument         : LPSTR buff
// Argument         : int bufflen
LPSTR CFirmwareFile::gets(LPSTR buff, int bufflen)
{
	if(m_iCurrPos >= m_FileInfo.nFileSizeLow)
		return NULL;


	LPSTR pEOL = strchr(&m_pFileData[m_iCurrPos],'\n');
	if(!pEOL)
		pEOL = m_pFileData+(m_FileInfo.nFileSizeLow);


	int XferLength = pEOL-(m_pFileData+m_iCurrPos);
	if(XferLength >= bufflen)
		XferLength = bufflen-1;


	memmove(buff,&m_pFileData[m_iCurrPos],XferLength);
	buff[XferLength] = 0;
	m_iCurrPos += (XferLength+1);

	return buff;

}
