// MessageSink.cpp: implementation of the CMessageSink class.
//
//////////////////////////////////////////////////////////////////////
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is MessageSink.cpp.
 *
 * The Initial Developer of the Original Code is
 * Fenestra Software LLC.
 * Portions created by the Initial Developer are Copyright (C) 2002 Fenestra Software LLC
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "stdafx.h"
#include "Phantom.h"
#include "MessageSink.h"
#include "PhantomCtrl.h"

#define MONITIMER (200)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMessageSink::CMessageSink(CPhantomCtrl *parent)
{
	m_pBrickCtl = parent;
	RECT rc = {0,0,0,0};
	Create(NULL,rc,_T("Message Notification Sink"),WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
}

CMessageSink::~CMessageSink()
{

}

// Function name	: CMessageSink::OnTimer
// Description	    : Calls PollPort to read a value from the RCX
// Return type		: LRESULT 
// Argument         : UINT uMsg
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
// Argument         : BOOL& bHandled
LRESULT CMessageSink::OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if(wParam == MONITIMER)
			m_pBrickCtl->OnMonitor(wParam);
		else
			m_pBrickCtl->PollPort(wParam);
		return 0;
	}


// Function name	: CMessageSink::OnDownloadProgress
// Description	    : Fires the DownloadProgress event.
// Return type		: LRESULT 
// Argument         : UINT uMsg
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
// Argument         : BOOL& bHandled
LRESULT CMessageSink::OnDownloadProgress(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		m_pBrickCtl->Fire_DownloadProgress(wParam);
		return 0;
	}


// Function name	: CMessageSink::OnDownloadDone
// Description	    : Fires the DownloadDone event.
// Return type		: LRESULT 
// Argument         : UINT uMsg
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
// Argument         : BOOL& bHandled
LRESULT CMessageSink::OnDownloadDone(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		m_pBrickCtl->Fire_DownloadDone(0,wParam);
		return 0;
	}


// Function name	: CMessageSink::OnThreadError
// Description	    : Fires the AsyncronBrickError event.
// Return type		: LRESULT 
// Argument         : UINT uMsg
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
// Argument         : BOOL& bHandled
LRESULT CMessageSink::OnThreadError(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		m_pBrickCtl->ReportError(wParam,(LPSTR)lParam);
		return 0;
	}


LRESULT CMessageSink::OnMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		m_pBrickCtl->Fire_PBMessage(wParam);
		return 0;
	}
