// PhantomCtrl.cpp : Implementation of CPhantomCtrl
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is PhantomCtrl.cpp.
 *
 * The Initial Developer of the Original Code is
 * Fenestra Software LLC.
 * Portions created by the Initial Developer are Copyright (C) 2002 Fenestra Software LLC
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"), 
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "stdafx.h"
#include "Phantom.h"
#include "PhantomCtrl.h"
#include "AboutDlg.h"
#include "ErrorCodes.h"
#include "RCX_Command.h"
 
// handy macros
#define LO_BYTE(s)	((BYTE)((s) & 0xFF))
#define HI_BYTE(s)	((BYTE)(((s) >> 8) & 0xFF))

#define RCX_VALUE(t, d)		(((int)(t) << 8) | ((int)(d) & 0xFF))
#define RCX_REPLY(c)		(~c & 0xFF)

#define LO_ADDBYTE(s)	((BYTE)((s) & 0x7F))
#define HI_ADDBYTE(s)	((BYTE)(((s) >> 7) & 0xFF))

const int TIMEROFFSET = 100;
const int MONITIMER = 200;

// Function name	: ReaderThreadProc
// Description	    : Global function for the comm port reader thread.
// Return type		: LONG 
// Argument         : LPVOID pParam
LONG ReaderThreadProc(LPVOID pParam)
{

	CPhantomCtrl *pCtrl = (CPhantomCtrl*)pParam;
	return pCtrl->Reader();
}



// Function name	: FirmwareThreadProc
// Description	    : Global function for the Firmware Downloader thread.
// Return type		: LONG 
// Argument         : LPVOID pParam
LONG FirmwareThreadProc(LPVOID pParam)
{

	CPhantomCtrl *pCtrl = (CPhantomCtrl*)pParam;
	return pCtrl->FirmwareDownloader();
}


// Function name	: ProgramThreadProc
// Description	    : Global function for the program downloader thread.
// Return type		: LONG 
// Argument         : LPVOID pParam
LONG ProgramThreadProc(LPVOID pParam)
{

	CPhantomCtrl *pCtrl = (CPhantomCtrl*)pParam;
	return pCtrl->ProgramDownloader();
}

/////////////////////////////////////////////////////////////////////////////
// CPhantomCtrl

// Function name	: CPhantomCtrl::get_ComPortNo
// Description	    : Returns ComPortNo
// Return type		: STDMETHODIMP 
// Argument         : COMPORTOPTIONS *pVal
STDMETHODIMP CPhantomCtrl::get_ComPortNo(COMPORTOPTIONS *pVal)
{
	*pVal = m_ComPortNo;

	return S_OK;
}


// Function name	: CPhantomCtrl::put_ComPortNo
// Description	    : Sets ComPortNo
// Return type		: STDMETHODIMP 
// Argument         : COMPORTOPTIONS newVal
STDMETHODIMP CPhantomCtrl::put_ComPortNo(COMPORTOPTIONS newVal)
{
	m_ComPortNo = newVal;
	SetDirty(TRUE);

	return S_OK;
}


// Function name	: CPhantomCtrl::get_LinkType
// Description	    : Gets the LinkType.
// Return type		: STDMETHODIMP 
// Argument         : LINKTYPEOPTIONS *pVal
STDMETHODIMP CPhantomCtrl::get_LinkType(LINKTYPEOPTIONS *pVal)
{
	*pVal = m_LinkType;

	return S_OK;
}


// Function name	: CPhantomCtrl::put_LinkType
// Description	    : Sets the LinkType
// Return type		: STDMETHODIMP 
// Argument         : LINKTYPEOPTIONS newVal
STDMETHODIMP CPhantomCtrl::put_LinkType(LINKTYPEOPTIONS newVal)
{
	m_LinkType = newVal;
	SetDirty(TRUE);

	return S_OK;
}


// Function name	: CPhantomCtrl::get_Brick
// Description	    : Gets the Brick type.
// Return type		: STDMETHODIMP 
// Argument         : PBRICKOPTIONS *pVal
STDMETHODIMP CPhantomCtrl::get_Brick(PBRICKOPTIONS *pVal)
{
	*pVal = m_Brick;

	return S_OK;
}


// Function name	: CPhantomCtrl::put_Brick
// Description	    : Sets the Brick type.
// Return type		: STDMETHODIMP 
// Argument         : PBRICKOPTIONS newVal
STDMETHODIMP CPhantomCtrl::put_Brick(PBRICKOPTIONS newVal)
{
	m_Brick = newVal;
	SetDirty(TRUE);
	return S_OK;
}


// Function name	: CPhantomCtrl::InitComm
// Description	    : Open the comm port and starts the reader thread.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::InitComm(VARIANT_BOOL *ReturnValue)
{
	BOOL fSuccess;

	*ReturnValue = VARIANT_TRUE;

	CommClose();

	m_hCom = CreateFile( m_CommPortStr[m_ComPortNo], 
		GENERIC_READ | GENERIC_WRITE, 
		0,						// comm devices must be opened w/exclusive-access
		NULL,					// no security attributes
		OPEN_EXISTING,			// comm devices must use OPEN_EXISTING
		FILE_FLAG_OVERLAPPED,	// overlapped I/O
		NULL					// hTemplate must be NULL for comm devices
		);

	if (m_hCom == INVALID_HANDLE_VALUE) {
		// Handle the error.
		ReportError(2001, GetLastError());
		*ReturnValue = VARIANT_FALSE;
		return S_OK;
	}


	if(m_ComPortNo != USB){
		DCB dcb;
		fSuccess = GetCommState(m_hCom, &dcb);

		if (!fSuccess) {
			// Handle the error.
			ReportError(2001, GetLastError());
			*ReturnValue = VARIANT_FALSE;
			return S_OK;
		}

		dcb.BaudRate = CBR_2400;	// set the baud rate
		dcb.ByteSize = 8;			// data size, xmit, and rcv
		dcb.Parity = ODDPARITY;		// no parity bit
		dcb.StopBits = ONESTOPBIT;	// one stop bit
		dcb.XonChar = 0;
		dcb.XoffChar = 0;
		dcb.XonLim = 0;
		dcb.XoffLim = 0;

		fSuccess = SetCommState(m_hCom, &dcb);

		if (!fSuccess) {
			// Handle the error.
			ReportError(2001, GetLastError());
			*ReturnValue = VARIANT_FALSE;
			return S_OK;
		}
	}

	// The reader thread fires this event when it has read and decoded a message from the RCX
	// When a command is sent to the RCX it waits on this event for an answer.
	m_hReadEvent = CreateEvent( 
        NULL, 			// no security attributes
		FALSE, 			// auto reset event 
		FALSE, 			// not signaled 
        "ReadEvent"		// object name
        );

	// Event used for overlaped IO on the com port.
	m_WriteOverlap.hEvent = CreateEvent(
		NULL,			// no security attributes 
		FALSE,			// auto reset event 
		FALSE,			// not signaled 
		NULL			// no name 
		);

	m_WriteOverlap.Offset = 0;
	m_WriteOverlap.OffsetHigh = 0;


	// Set the comm port timeouts to something reasonable.
	COMMTIMEOUTS TimeOut;

	GetCommTimeouts(m_hCom, &TimeOut);
	TimeOut.ReadIntervalTimeout = 100;
	TimeOut.ReadTotalTimeoutMultiplier = 30;
	TimeOut.ReadTotalTimeoutConstant = 200;
	TimeOut.WriteTotalTimeoutMultiplier = 30;
	TimeOut.WriteTotalTimeoutConstant = 200;
	SetCommTimeouts(m_hCom, &TimeOut);

	// Start the reader thread.
	if((m_hReaderThread = CreateThread(NULL, 
			0, 
			(LPTHREAD_START_ROUTINE)ReaderThreadProc, 
			(void *)this, 
			0, 
			&m_ReaderThreadID)) == NULL){
		ReportError(2001, GetLastError());
		*ReturnValue = VARIANT_FALSE;
		return S_OK;         
	}

	return S_OK;
}


// Function name	: CPhantomCtrl::CloseComm
// Description	    : Close the Comm Port
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::CloseComm(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_TRUE;
	CommClose();
	return S_OK;
}


// Function name	: CPhantomCtrl::IgnDLerrUntilGoodAnswer
// Description	    : 
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::IgnDLerrUntilGoodAnswer(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	return S_OK;
}


// Function name	: CPhantomCtrl::GetRetransmitStatistics
// Description	    : Get the RetransmitStatistics formated into a safearray
// Return type		: void 
// Argument         : CRetryCounter *ErrorCtr
// Argument         : VARIANT *ReturnValue
void CPhantomCtrl::GetRetransmitStatistics(CRetryCounter *ErrorCtr, VARIANT *ReturnValue)
{
	int var;
	long ndex[2];
	SAFEARRAY * pSA;
	// Create the safearray.
	SAFEARRAYBOUND sabRSarray[2];	// A two-dimensional array.
	sabRSarray[0].cElements=2;		// 
	sabRSarray[0].lLbound=0;		//Lower bound of array.
	sabRSarray[1].cElements=12;		//
	sabRSarray[1].lLbound=0;		//Lower bound of array.

	pSA=SafeArrayCreate(VT_I4, 2, sabRSarray);

	ndex[1] = 0;
	ndex[0] = 0;
	var = ErrorCtr->m_Total;
	SafeArrayPutElement(pSA, ndex, &var);

	ndex[0] = 1;
	var = ErrorCtr->m_Total;
	SafeArrayPutElement(pSA, ndex, &var);

	for(ndex[1] = 1; ndex[1] <= 11; ndex[1]++){
		ndex[0] = 0;
		var = ErrorCtr->m_Retries[ndex[1]-1];
		SafeArrayPutElement(pSA, ndex, &var);

		ndex[0] = 1;
		var = ErrorCtr->m_Retries[ndex[1]-1];
		SafeArrayPutElement(pSA, ndex, &var);
	}

	// Load the safearray into an output variable.
	ReturnValue->vt = VT_ARRAY|VT_I4;
	ReturnValue->parray = pSA;
}


// Function name	: CPhantomCtrl::GetShortTermRetransStatistics
// Description	    : Get the short term retrans, and then clear them.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT *ReturnValue
STDMETHODIMP CPhantomCtrl::GetShortTermRetransStatistics(VARIANT *ReturnValue)
{
	GetRetransmitStatistics(&m_ShortTermRetryCtr, ReturnValue);
	m_ShortTermRetryCtr.Clear();
	return S_OK;
}


// Function name	: CPhantomCtrl::GetLongTermRetransmitStatistics
// Description	    : Get the long term retrans.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT *ReturnValue
STDMETHODIMP CPhantomCtrl::GetLongTermRetransmitStatistics(VARIANT *ReturnValue)
{
	GetRetransmitStatistics(&m_LongTermRetryCtr, ReturnValue);
	return S_OK;
}


// Function name	: CPhantomCtrl::PBAliveOrNot
// Description	    : Check that the RCX is alive.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::PBAliveOrNot(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	RCX_Command cmd(pbPBAliveOrNot);

	BYTE pReply[5];
	DWORD ReplyLeng;
	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPBAliveOrNot))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::TowerAlive
// Description	    : Check if the Tower is alive
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::TowerAlive(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	PBAliveOrNot(ReturnValue);
	if(m_ComPortNo != USB){
		if(m_bEchoEvent == true)
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::TowerAndCableConnected
// Description	    : Check that the tower is connected.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::TowerAndCableConnected(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	DWORD modemstatus;
	if(m_ComPortNo == USB){
		*ReturnValue = VARIANT_TRUE;
	}
	else {
		EscapeCommFunction(m_hCom, SETRTS);
		GetCommModemStatus(m_hCom, &modemstatus);
		if(modemstatus == MS_CTS_ON){
			*ReturnValue = VARIANT_TRUE;
		}
		EscapeCommFunction(m_hCom, CLRRTS);
	}

	return S_OK;
}


// Function name	: CPhantomCtrl::AlterDir
// Description	    : Change the direction of the motor(s)
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::AlterDir(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){
		ml |= DirAlt;

		RCX_Command cmd(pbSetFwdSetRwdRewDir, ml);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbOnOffFloat))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::ClearTachoCounter
// Description	    : Not Used (cybermaster)
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::ClearTachoCounter(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	// CyberMaster Command - Not Implemented
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	return S_OK;
}


// Function name	: CPhantomCtrl::Drive
// Description	    : Not Used (cybermaster)
// Return type		: STDMETHODIMP 
// Argument         : short Number0
// Argument         : short Number1
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::Drive(short Number0, short Number1, VARIANT_BOOL *ReturnValue)
{
	// CyberMaster Command - Not Implemented
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	return S_OK;
}


// Function name	: CPhantomCtrl::Float
// Description	    : Put the motor(s) into float.
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::Float(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){
		ml |= MotFlt;

		RCX_Command cmd(pbOnOffFloat, ml);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbOnOffFloat))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;}


// Function name	: CPhantomCtrl::Off
// Description	    : Turn the motor(s) off.
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::Off(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){
		ml |= MotOff;

		RCX_Command cmd(pbOnOffFloat, ml);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbOnOffFloat))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::On
// Description	    : Turn the motor(s) on.
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::On(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	


	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){
		ml |= MotOn;

		RCX_Command cmd(pbOnOffFloat, ml);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbOnOffFloat))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::OnWait
// Description	    : CyberMaster Command - Not Implemented
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : short Number
// Argument         : short Time
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::OnWait(BSTR MotorList, short Number, short Time, VARIANT_BOOL *ReturnValue)
{
	// CyberMaster Command - Not Implemented
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	return S_OK;
}


// Function name	: CPhantomCtrl::OnWaitDifferent
// Description	    : CyberMaster Command - Not Implemented
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : short Number0
// Argument         : short Number1
// Argument         : short Number2
// Argument         : short Time
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::OnWaitDifferent(BSTR MotorList, short Number0, short Number1, short Number2, short Time, VARIANT_BOOL *ReturnValue)
{
	// CyberMaster Command - Not Implemented
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	return S_OK;
}


// Function name	: CPhantomCtrl::SetFwd
// Description	    : Set the motor(s) to forward.
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetFwd(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	


	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){
		ml |= DirFwd;

		RCX_Command cmd(pbSetFwdSetRwdRewDir, ml);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbOnOffFloat))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::SetPower
// Description	    : Set the motor(s) power level.
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetPower(BSTR MotorList, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){

		RCX_Command cmd(pbSetPower, ml, (BYTE)Source, (BYTE)Number);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbSetPower))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::SetRwd
// Description	    : Set the motor(s) to reverse.
// Return type		: STDMETHODIMP 
// Argument         : BSTR MotorList
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetRwd(BSTR MotorList, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	BYTE ml;
	if(ParseMotorList(MotorList, &ml)){
		ml |= DirRev;

		RCX_Command cmd(pbSetFwdSetRwdRewDir, ml);

		if(m_pProgramBuffer){
			m_pProgramBuffer->AddData(cmd);
			*ReturnValue = VARIANT_TRUE;
			return S_OK;
		}
		SendData(cmd, pReply, &ReplyLeng);
		if(pReply[0] == RCX_REPLY(pbOnOffFloat))
			*ReturnValue = VARIANT_TRUE;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::Wait
// Description	    : Wait for an event.
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::Wait(RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	if(!m_pProgramBuffer)
		return S_OK;

	RCX_Command cmd(pbWait, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	m_pProgramBuffer->AddData(cmd);
	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::AbsVar
// Description	    : Absolute Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::AbsVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	RCX_Command cmd(pbAbsVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));
	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbAbsVar))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::AndVar
// Description	    : And Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::AndVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	RCX_Command cmd(pbAndVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));
	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbAndVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::DivVar
// Description	    : Divide Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DivVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbDivVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbDivVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::MulVar
// Description	    : Multiply Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::MulVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbMulVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbMulVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::OrVar
// Description	    : Or Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::OrVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbOrVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbOrVar))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::SetVar
// Description	    : Set Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbSetVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSetVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SgnVar
// Description	    : 
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SgnVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	
	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbSgnVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSgnVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SubVar
// Description	    : Subtract Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SubVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	
	
	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbSubVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSubVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SumVar
// Description	    : Add Value of Variable
// Return type		: STDMETHODIMP 
// Argument         : short VarNo
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SumVar(short VarNo, RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbSumVar, (BYTE)VarNo, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSumVar))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::DatalogNext
// Description	    : Nest Datalog entry
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DatalogNext(RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbDataLogNext, (BYTE)Source, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbDataLogNext))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::SetDatalog
// Description	    : Set the datalog size
// Return type		: STDMETHODIMP 
// Argument         : short Size
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetDatalog(short Size, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbSetDataLog, LO_BYTE(Size), HI_BYTE(Size));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSetDataLog))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::UploadDatalog
// Description	    : Fetch the datalog fromthe RCX
// Return type		: STDMETHODIMP 
// Argument         : short From
// Argument         : short Size
// Argument         : VARIANT *ReturnValue
STDMETHODIMP CPhantomCtrl::UploadDatalog(short From, short Size, VARIANT *ReturnValue)
{
	VariantClear(ReturnValue);
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[255];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbUpload, LO_BYTE(From), HI_BYTE(From), LO_BYTE(Size), HI_BYTE(Size));

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbUpload)){
		int var;
		long ndex[3];
		SAFEARRAY * pSA;
		// Create the safearray.
		SAFEARRAYBOUND sabRSarray[2];	// A two-dimensional array.
		sabRSarray[0].cElements=3;		// 
		sabRSarray[0].lLbound=0;		//Lower bound of array.
		sabRSarray[1].cElements=Size;	//
		sabRSarray[1].lLbound=0;		//Lower bound of array.

		pSA=SafeArrayCreate(VT_I4, 2, sabRSarray);
		ndex[1] = 0;
		for(unsigned int x=1;x < ReplyLeng;x+=3, ndex[1]++){
			ndex[0]=0;
			if(x == 1){
				var = pReply[x];
				SafeArrayPutElement(pSA, ndex, &var);
				ndex[0]=1;
				var = 0; 
				SafeArrayPutElement(pSA, ndex, &var);
			}
			else
			{
				switch(pReply[x]>>5){
				case 0:
					var = 0;
					break;
				case 1:
					var = 1;
					break;
				case 2:
					var = 9;
					break;
				case 4:
					var = 14;
				}
				SafeArrayPutElement(pSA, ndex, &var);
				ndex[0]=1;
				var = pReply[x] & 0x1F; 
				SafeArrayPutElement(pSA, ndex, &var);
			}
			ndex[0]=2;
			var = RCX_VALUE(pReply[x+2], pReply[x+1]);
			SafeArrayPutElement(pSA, ndex, &var);
		}


		// Load the safearray into an output variable.
		ReturnValue->vt = VT_ARRAY|VT_I4;
		ReturnValue->parray = pSA;
	}
	return S_OK;
}


// Function name	: CPhantomCtrl::ClearAllEvents
// Description	    : Clear All Events
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::ClearAllEvents(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbClearAllEvents);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbClearAllEvents))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::ClearEvent
// Description	    : Clear a Polling event.
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::ClearEvent(RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	int i;
	for(i = 0; i < MAXEVENTS; i++){
		if(m_Target[i].m_TimerID && (m_Target[i].m_Source == Source) && (m_Target[i].m_Number == Number)){
		m_pMessageSink->KillTimer(m_Target[i].m_TimerID);
		m_Target[i].m_TimerID = 0;
		*ReturnValue = VARIANT_TRUE;
		}
	}

	return S_OK;
}


// Function name	: CPhantomCtrl::MemMap
// Description	    : Get the Memory Map of the RCX
// Return type		: STDMETHODIMP 
// Argument         : VARIANT *ReturnValue
STDMETHODIMP CPhantomCtrl::MemMap(VARIANT *ReturnValue)
{
	VariantClear(ReturnValue);
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[255];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbMemMap);

	SendData(cmd, pReply, &ReplyLeng);

	int var;
	long ndex;
	SAFEARRAY * pSA;
	// Create the safearray.
	SAFEARRAYBOUND sabRSarray[1];	// A one-dimensional array.
	sabRSarray[0].cElements=95;		// 94 elements.
	sabRSarray[0].lLbound=0;		//Lower bound of array.

	pSA=SafeArrayCreate(VT_I4, 1, sabRSarray);
	ndex = 0;

	if(pReply[0] == RCX_REPLY(pbMemMap)){
		var = pReply[0];
		SafeArrayPutElement(pSA, &ndex, &var);

		ndex = 1;
		for(unsigned int x=1;x < ReplyLeng;x+=2, ndex++){
			var = RCX_VALUE(pReply[x], pReply[x+1]);
			AtlTrace("%d %d %x %x %x\n", x, var, var, pReply[x], pReply[x+1]);
			SafeArrayPutElement(pSA, &ndex, &var);
		}
	}
	else {
		var = 0;
		SafeArrayPutElement(pSA, &ndex, &var);
	}


	// Load the safearray into an output variable.
	ReturnValue->vt = VT_ARRAY|VT_I4;
	ReturnValue->parray = pSA;
	return S_OK;
}


// Function name	: CPhantomCtrl::PBBattery
// Description	    : Get the Battery voltage.
// Return type		: STDMETHODIMP 
// Argument         : short *ReturnValue
STDMETHODIMP CPhantomCtrl::PBBattery(short *ReturnValue)
{
	*ReturnValue = 0;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbPBBattery);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPBBattery))
		*ReturnValue = RCX_VALUE(pReply[2], pReply[1]);

	return S_OK;
}


// Function name	: CPhantomCtrl::Poll
// Description	    : Read a variable from the RCX
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : short *ReturnValue
STDMETHODIMP CPhantomCtrl::Poll(RCX_SOURCE Source, short Number, short *ReturnValue)
{
	*ReturnValue = 0;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;
	
	RCX_Command cmd(pbPoll, BYTE(Source), BYTE(Number));

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPoll))
		*ReturnValue = RCX_VALUE(pReply[2], pReply[1]);

	return S_OK;
}


// Function name	: CPhantomCtrl::SetEvent
// Description	    : Set a timer to poll the rcx for a value
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : short Time
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetEvent(RCX_SOURCE Source, short Number, short Time, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	int i;
	for(i = 0; i < MAXEVENTS; i++){
		if(m_Target[i].m_TimerID && (m_Target[i].m_Source == Source) && (m_Target[i].m_Number == Number)){
			m_pMessageSink->KillTimer(m_Target[i].m_TimerID);
			m_Target[i].m_TimerID = 0;
		}
	}

	for(i = 0; i < MAXEVENTS; i++){
		if(m_Target[i].m_TimerID == 0)
			break;
	}

	if(i != MAXEVENTS){
		m_Target[i].m_Source = Source;
		m_Target[i].m_Number = Number;
		m_Target[i].m_TimerID = m_pMessageSink->SetTimer(TIMEROFFSET+i, (UINT)Time);
		if(m_Target[i].m_TimerID)
			*ReturnValue = VARIANT_TRUE;
	}

	return S_OK;
}


// Function name	: CPhantomCtrl::ClearPBMessage
// Description	    : Clear the PB Message buffer
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::ClearPBMessage(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbClearPBMessage);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbClearPBMessage))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::ClearTimer
// Description	    : Clear the RCX Timer
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::ClearTimer(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbClearTimer, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbClearTimer))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::PBPowerdownTime
// Description	    : Set the Powerdown time for the RCX
// Return type		: STDMETHODIMP 
// Argument         : short Time
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::PBPowerdownTime(short Time, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPBPowerDownTime, (BYTE)Time);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPBPowerDownTime))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::PBTurnOff
// Description	    : Powerdown the RCX
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::PBTurnOff(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPBTurnOff);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPBTurnOff))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::PBTxPower
// Description	    : Set the xmit power level in the RCX
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::PBTxPower(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPbTXPower, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPbTXPower))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::PlaySystemSound
// Description	    : Play a sound
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::PlaySystemSound(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPlaySystemSound, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPlaySystemSound))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::PlayTone
// Description	    : Play a tone
// Return type		: STDMETHODIMP 
// Argument         : short Frequency
// Argument         : short Time
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::PlayTone(short Frequency, short Time, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPlayTone, LO_BYTE(Frequency), HI_BYTE(Frequency), (BYTE)Time);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPlayTone))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::SelectDisplay
// Description	    : Show stuff in the LDC window
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SelectDisplay(RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbSelectDisplay, (BYTE)Source, LO_BYTE(Number), HI_BYTE(Number));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSelectDisplay))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SendPBMessage
// Description	    : Send a Message to an RCX.
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SendPBMessage(RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbSendPBMessage, (BYTE)Source, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSendPBMessage))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SetWatch
// Description	    : Sets the internal watch to the given time.
// Return type		: STDMETHODIMP 
// Argument         : short Hours
// Argument         : short Min
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetWatch(short Hours, short Min, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbSetWatch, (BYTE)Hours, (BYTE)Min);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSetWatch))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::ClearSensorValue
// Description	    : Resets the source 9 register for the sensor.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::ClearSensorValue(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbClearSensorValue, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbClearSensorValue))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SetSensorMode
// Description	    : Sets the given mode and measurement slope for the given sensor.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : short Mode
// Argument         : short Slope
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetSensorMode(short Number, short Mode, short Slope, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbSetSensorMode, (BYTE)Number, (BYTE)((Mode << 5)  | (Slope & 0x1F)));

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSetSensorMode))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::SetSensorType
// Description	    : Sets the given sensor to the given type.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : short Type
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SetSensorType(short Number, short Type, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbSetSensorType, (BYTE)Number, (BYTE)Type);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSetSensorType))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::DownloadFirmware
// Description	    : Sets up to download new firmware to the RCX
// Return type		: STDMETHODIMP 
// Argument         : BSTR Filename
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DownloadFirmware(BSTR Filename, VARIANT_BOOL *ReturnValue)
{

	if(CheckConnection()==false)
		return S_OK;	

	USES_CONVERSION;

	CFirmwareFile ffile;

	*ReturnValue = VARIANT_FALSE;

	//If the m_pFirmwareData exist, we are already downloading firmware.

	if(m_pFirmwareData){
		ReportError(2016, "Download in progress");
		return S_OK;
	}

	//Create a new CFirmwareData
	m_pFirmwareData = new CFirmwareData;
	if(m_pFirmwareData->Load(OLE2A(Filename), this)==false){
		return S_OK;
	}
	// Tell the world.
	Fire_DownloadStatus(m_pFirmwareData->length*20, m_pFirmwareData->length, (SHORT)100);

	// Start a new thread to download the data to the brick.
	if((m_hFirmwareThread = CreateThread(NULL, 
			0, 
			(LPTHREAD_START_ROUTINE)FirmwareThreadProc, 
			(void *)this, 
			0, 
			&m_FirmwareThreadID)) == NULL){
		ReportError(2016, GetLastError());
		return S_OK;         
	}

	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::ProgramDownloader
// Description	    : Downloads a new Task or Sub to the RCX
// Return type		: LONG 
LONG CPhantomCtrl::ProgramDownloader()
{
	int xferzise = PROGRAM_TRANSFER_SIZE;
	int addr;
	int index;
	BYTE status;
	BYTE *ProgramImage;
	CComBSTR xretval;
	bool startstatus;

	if(m_pProgramBuffer->GetType() == TYPE_SUB){
		startstatus = BeginSubDownload(m_pProgramBuffer->GetNumber(), m_pProgramBuffer->GetLength());
	}
	else {
		startstatus = BeginTaskDownload(m_pProgramBuffer->GetNumber(), m_pProgramBuffer->GetLength());
	}
	if(startstatus == false)
		return 0;

	//Send the program in TRANSFER_SIZE blocks

	ProgramImage=m_pProgramBuffer->GetData();

	for (addr = 0, index = 1; addr < m_pProgramBuffer->GetLength(); addr += xferzise, index++) {
		//Check for last block.
		if((addr + xferzise) > m_pProgramBuffer->GetLength()){
			xferzise = m_pProgramBuffer->GetLength() - addr;
			index = 0;	// last block, is always zero
		}

		if(ContinueFirmwareDownLoad(index, xferzise, &ProgramImage[addr], &status)){
			m_pMessageSink->SendMessage(PBR_DOWNLOADPROG , addr);
		}
		else {
			//End the download.
			m_pMessageSink->SendMessage(PBR_DOWNLOADDONE , m_FirmwareThreadID);
			return 0;
		}
	}

	//Send the last progress message.
	m_pMessageSink->SendMessage(PBR_DOWNLOADPROG , addr);

	m_pMessageSink->SendMessage(PBR_DOWNLOADDONE , m_FirmwareThreadID);
	return 0;
}


// Function name	: CPhantomCtrl::FirmwareDownloader
// Description	    : Download new firmware to the RCX
// Return type		: LONG 
LONG CPhantomCtrl::FirmwareDownloader()
{
	int xferzise = TRANSFER_SIZE;
	int addr;
	int index;
	BYTE status;
	CComBSTR xretval;

	if(m_ComPortNo == USB){
		xferzise = USB_TRANSFER_SIZE;
	}

	//Boot the brick.
	GoIntoBootMode();

	//Send the start address and checksum.
	BeginFirmwareDownLoad(m_pFirmwareData->iStart, m_pFirmwareData->GetChecksum());

	//Send the firmware in TRANSFER_SIZE blocks
	for (addr = 0, index = 1; addr < m_pFirmwareData->length; addr += xferzise, index++) {
		//Check for last block.
		if((addr + xferzise) > m_pFirmwareData->length){
			xferzise = m_pFirmwareData->length - addr;
			index = 0;	// last block, is always zero
		}
		if(ContinueFirmwareDownLoad(index, xferzise, &m_pFirmwareData->Image[addr], &status)){
			m_pMessageSink->SendMessage(PBR_DOWNLOADPROG , addr);
		}
		else {
			//End the download.
			delete m_pFirmwareData;
			m_pFirmwareData = NULL;
			m_pMessageSink->SendMessage(PBR_DOWNLOADDONE , m_FirmwareThreadID);
			return 0;
		}
	}
	//Send the last progress message.
	m_pMessageSink->SendMessage(PBR_DOWNLOADPROG , addr);

	//I don't know if these PBAliveOrNot's are really needed, but the 
	//company software does it.
	VARIANT_BOOL z;
	PBAliveOrNot(&z);

	//Unlock the firmware.
	CComBSTR UnlockString = "Just a bit off the block!";
	UnlockFirmware(UnlockString, &xretval);

	PBAliveOrNot(&z);
	PBAliveOrNot(&z);

	//Unlock the brick.
	UnlockPBrick(&xretval);

	//The download.is done.
	delete m_pFirmwareData;
	m_pFirmwareData = NULL;

	m_pMessageSink->SendMessage(PBR_DOWNLOADDONE , m_FirmwareThreadID);
	//Leave this thread.
	return 0;
}



// Function name	: CPhantomCtrl::UnlockFirmware
// Description	    : Unlocks the firmware in the brick
// Return type		: STDMETHODIMP 
// Argument         : BSTR UnlockString
// Argument         : BSTR *ReturnValue
STDMETHODIMP CPhantomCtrl::UnlockFirmware(BSTR UnlockString, BSTR *ReturnValue)
{
	CComBSTR cbretval = "The Control OCX can not get a valid PBrick Unlock string!";
	*ReturnValue = cbretval.Copy();
	if(CheckConnection()==false)
		return S_OK;	

	CComBSTR cbUnlockString;
	CComBSTR retstring;

	BYTE pReply[255];
	DWORD ReplyLeng;
	cbUnlockString.Attach(UnlockString);

	RCX_Command cmd(pbUnlockFirmware, 0x4C, 0x45, 0x47, 0x4F, 0xAE);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbUnlockFirmware)){
		retstring=(char *)&pReply[1];
		if(cbUnlockString == retstring)
			cbretval = "This is a Control OCX communicating with a LEGO PBrick!";
	}
	else
	{
		cbretval = "Unlock failed";
	}

	*ReturnValue = cbretval.Copy();

	cbUnlockString.Detach();
	return S_OK;
}


// Function name	: CPhantomCtrl::UnlockPBrick
// Description	    : Unlocks the brick
// Return type		: STDMETHODIMP 
// Argument         : BSTR *ReturnValue
STDMETHODIMP CPhantomCtrl::UnlockPBrick(BSTR *ReturnValue)
{
	CComBSTR FirmID ="";
	*ReturnValue = FirmID.Copy();
	if(CheckConnection()==false)
		return S_OK;	


	char aFirmID[12];

	BYTE pReply[255];
	DWORD ReplyLeng;

	RCX_Command cmd(pbUnlockPBrick, 1, 3, 5, 7, 11);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbUnlockPBrick)){
		wsprintf(aFirmID, "%d%d.%d%d/%d%d.%d%d", pReply[1], pReply[2], pReply[3], pReply[4]
											  , pReply[5], pReply[6], pReply[7], pReply[8]);
		//  03.01/03.28
		FirmID = aFirmID;
		*ReturnValue = FirmID.Copy();
	}
	return S_OK;}


// Function name	: CPhantomCtrl::DeleteAllSubs
// Description	    : The command deletes all subroutines in the currently selected program.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DeleteAllSubs(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbDeleteAllSubs);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbDeleteAllSubs))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::DeleteAllTasks
// Description	    : The command deletes all Tasks in the currently selected program.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DeleteAllTasks(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbDeleteAllTasks);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbDeleteAllTasks))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::DeleteSub
// Description	    : The command deletes a subroutines in the currently selected program.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DeleteSub(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbDeleteSub, (BYTE)Number);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbDeleteSub))
		*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::DeleteTask
// Description	    : The command deletes a Task in the currently selected program.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::DeleteTask(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbDeleteTask, (BYTE)Number);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbDeleteTask))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::SelectPrgm
// Description	    : The command stops all tasks in the currently selected program and then selects the given
//					: program slot.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::SelectPrgm(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbSelectProgram, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbSelectProgram))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::GoSub
// Description	    : Calls a subroutine
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::GoSub(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_pProgramBuffer->GetType() == TYPE_SUB){
		CComBSTR Description;
		Description.LoadString(SYNTAX_GOSUB_NOT_ALLOWED_IN_SUBS);
		Fire_AsyncronBrickError(SYNTAX_GOSUB_NOT_ALLOWED_IN_SUBS, Description);
		return S_OK;
	}


	RCX_Command cmd(pbGosub, (BYTE)Number);

	m_pProgramBuffer->AddData(cmd);
	*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::StartTask
// Description	    : Starts a specified task
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::StartTask(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbStartTask, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbStartTask))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::StopAllTasks
// Description	    : Stop all tasks
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::StopAllTasks(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbStopAllTasks);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbStopAllTasks))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::StopTask
// Description	    : Stop a task
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::StopTask(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbStopTask, (BYTE)Number);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbStopTask))
		*ReturnValue = VARIANT_TRUE;


	return S_OK;
}


// Function name	: CPhantomCtrl::BeginOfTask
// Description	    : Marks the begining of a task. We start buffering all calls
//					: and build a task to download.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::BeginOfTask(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(m_pProgramBuffer)
		return S_OK;

	m_pProgramBuffer = new ProgramData;
	m_pProgramBuffer->SetType(TYPE_TASK);
	m_pProgramBuffer->SetNumber(Number);

	return S_OK;
}


// Function name	: CPhantomCtrl::EndOfTask
// Description	    : Starts the download of a new task.
// Return type		: STDMETHODIMP 
// Argument         : short *ReturnValue
STDMETHODIMP CPhantomCtrl::EndOfTask(short *ReturnValue)
{
	*ReturnValue = 0;
	if(CheckConnection()==false)
		return S_OK;	

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_pProgramBuffer->GetType() == TYPE_SUB){
		CComBSTR Description;
		Description.LoadString(SYNTAX_ENDOFTASK_RECIEVED_ENDOFSUB_EXPECTED);
		Fire_AsyncronBrickError(SYNTAX_ENDOFTASK_RECIEVED_ENDOFSUB_EXPECTED, Description);
		return S_OK;
	}
	if(!m_IfElseStack.empty() || !m_WhileLoopStack.empty()){
		CComBSTR Description;
		Description.LoadString(SYNTAX_END_REACHED_TOO_SOON);
		Fire_AsyncronBrickError(SYNTAX_END_REACHED_TOO_SOON, Description);
		return S_OK;
	}

	// Tell the world.
	Fire_DownloadStatus((m_pProgramBuffer->GetLength()*20), m_pProgramBuffer->GetLength(), (SHORT)m_pProgramBuffer->GetNumber());

	if((m_hFirmwareThread = CreateThread(NULL, 
			0, 
			(LPTHREAD_START_ROUTINE)ProgramThreadProc, 
			(void *)this, 
			0, 
			&m_FirmwareThreadID)) == NULL){
		ReportError(2016, GetLastError());
		return S_OK;         
	}

	AtlWaitWithMessageLoop(m_hFirmwareThread);
//	DWORD Waiting = WAIT_TIMEOUT;
//	while(Waiting==WAIT_TIMEOUT){
//		Waiting = WaitForSingleObject(m_hFirmwareThread, 2000);
//		AtlTrace("Waiting = %d\n", Waiting);
//	}

	delete m_pProgramBuffer;
	m_pProgramBuffer = NULL;

	return S_OK;
}


// Function name	: CPhantomCtrl::EndOfTaskNoDownload
// Description	    : Fires the DownloadStatus so the VB programs can see the program size.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::EndOfTaskNoDownload(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	Fire_DownloadStatus((m_pProgramBuffer->GetLength()*20), m_pProgramBuffer->GetLength(), m_pProgramBuffer->GetNumber());	

	return S_OK;
}


// Function name	: CPhantomCtrl::BeginOfSub
// Description	    : Marks the begining of a Sub. We start buffering all calls
//					: and build a Sub to download.
// Return type		: STDMETHODIMP 
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::BeginOfSub(short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(m_pProgramBuffer)
		return S_OK;

	m_pProgramBuffer = new ProgramData;
	m_pProgramBuffer->SetType(TYPE_SUB);
	m_pProgramBuffer->SetNumber(Number);

	return S_OK;
}


// Function name	: CPhantomCtrl::EndOfSub
// Description	    : Starts the download of a new sub.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::EndOfSub(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_pProgramBuffer->GetType() == TYPE_TASK){
		CComBSTR Description;
		Description.LoadString(SYNTAX_ENDOFSUB_RECIEVED_ENDOFTASK_EXPECTED);
		Fire_AsyncronBrickError(SYNTAX_ENDOFSUB_RECIEVED_ENDOFTASK_EXPECTED, Description);
		return S_OK;
	}

	if(!m_IfElseStack.empty() || !m_WhileLoopStack.empty()){
		CComBSTR Description;
		Description.LoadString(SYNTAX_END_REACHED_TOO_SOON);
		Fire_AsyncronBrickError(SYNTAX_END_REACHED_TOO_SOON, Description);
		return S_OK;
	}

	ExitSub(ReturnValue);

	// Tell the world.
	Fire_DownloadStatus((m_pProgramBuffer->GetLength()*20), m_pProgramBuffer->GetLength(), (SHORT)m_pProgramBuffer->GetNumber()+10);

	if((m_hFirmwareThread = CreateThread(NULL, 
			0, 
			(LPTHREAD_START_ROUTINE)ProgramThreadProc, 
			(void *)this, 
			0, 
			&m_FirmwareThreadID)) == NULL){
		ReportError(2016, GetLastError());
		return S_OK;         
	}

	AtlWaitWithMessageLoop(m_hFirmwareThread);
//	DWORD Waiting = WAIT_TIMEOUT;
//	while(Waiting==WAIT_TIMEOUT){
//		Waiting = WaitForSingleObject(m_hFirmwareThread, 2000);
//		AtlTrace("Waiting = %d\n", Waiting);
//	}

	delete m_pProgramBuffer;
	m_pProgramBuffer = NULL;

	return S_OK;
}


// Function name	: CPhantomCtrl::EndOfSubNoDownload
// Description	    : Fires the DownloadStatus so the VB programs can see the program size.
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::EndOfSubNoDownload(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	Fire_DownloadStatus((m_pProgramBuffer->GetLength()*20), m_pProgramBuffer->GetLength(), m_pProgramBuffer->GetNumber()+10);

	return S_OK;
}

// Function name	: CPhantomCtrl::Loop
// Description	    : Starts a loop in the program to download
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source
// Argument         : short Number
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::Loop(RCX_SOURCE Source, short Number, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	// Special case - Loop Forever
	if((Source == SRC_CON) & (Number == 0)){
		m_WhileLoopStack.push(m_pProgramBuffer->GetLength());	// Save the loop start address
		m_WhileLoopStack.push(-1);
		return S_OK;
	}

	RCX_Command cmd(pbSetLoopCounter, (BYTE)Source, (BYTE)Number);

	m_pProgramBuffer->AddData(cmd);
	m_WhileLoopStack.push(m_pProgramBuffer->GetLength());	// Save the loop start address

	m_WhileLoopStack.push(m_pProgramBuffer->GetLength()+1);	// Push the address of the address to end the loop
	cmd.Set(pbLCheckLoopCounter, (BYTE)0, (BYTE)0);
	m_pProgramBuffer->AddData(cmd);

	*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


// Function name	: CPhantomCtrl::EndLoop
// Description	    : Ends the loop
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::EndLoop(VARIANT_BOOL *ReturnValue)
{
	BYTE pBuff[5];
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_WhileLoopStack.empty()){
		CComBSTR Description;
		Description.LoadString(SYNTAX_MATCHING_ENDLOOP_NOT_FOUND);
		Fire_AsyncronBrickError(SYNTAX_MATCHING_ENDLOOP_NOT_FOUND, Description);
		return S_OK;
	}

	int exitAddress = m_WhileLoopStack.top();
	m_WhileLoopStack.pop();

	int startAddress = m_WhileLoopStack.top();
	m_WhileLoopStack.pop();

	int distance;

	// Special case of loop forever
	if(exitAddress == -1){
		// Calculate the distance to the start of the loop.
		distance = (m_pProgramBuffer->GetLength()+1) - startAddress;

		if(distance <= 0x7F){
			pBuff[0] = pbSJump;
			pBuff[1] = 0x80|LO_ADDBYTE(distance);
			m_pProgramBuffer->AddData(pBuff);
		}
		else {
			pBuff[0] = pbLJump;
			pBuff[1] = 0x80|LO_ADDBYTE(distance);
			pBuff[2] = HI_ADDBYTE(distance);
			m_pProgramBuffer->AddData(pBuff);
		}

		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}

	// Calculate the distance to the start of the loop.

	distance = (m_pProgramBuffer->GetLength()+1) - startAddress;

	RCX_Command cmd(pbLJump, 0x80|LO_ADDBYTE(distance), HI_ADDBYTE(distance));

	m_pProgramBuffer->AddData(cmd);

	// Calculate the distance to the end of the loop.

	distance = (m_pProgramBuffer->GetLength()) - exitAddress;
	// Get us a pointer to the program data
	BYTE *pBuffer = m_pProgramBuffer->GetData();
	// fill in the address of the end of the loop
	pBuffer[exitAddress] = LO_ADDBYTE(distance);
	pBuffer[exitAddress+1] = HI_ADDBYTE(distance);

	m_pProgramBuffer->DebugDump();
	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::While
// Description	    : Starts a While loop.
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source1
// Argument         : short Number1
// Argument         : short RelOp
// Argument         : RCX_SOURCE Source2
// Argument         : short Number2
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::While(RCX_SOURCE Source1, short Number1, short RelOp, RCX_SOURCE Source2, short Number2, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	RCX_Command cmd(pbLCheckDo, (BYTE)Source1 | (RelOp<<6), (BYTE)Source2, LO_BYTE(Number1), HI_BYTE(Number1), (BYTE)Number2);

	m_WhileLoopStack.push(m_pProgramBuffer->GetLength());	// Push the address of the start of the loop
	m_pProgramBuffer->AddData(cmd);

	BYTE pBuff[3];
	pBuff[0] = 1;											// Add two bytes for the addres we fill in later.
	pBuff[1] = (BYTE)0xFF;

	m_WhileLoopStack.push(m_pProgramBuffer->GetLength());	// Push the address of the address to end the loop
	m_pProgramBuffer->AddData(pBuff);

	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::EndWhile
// Description	    : Ends the while loop
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::EndWhile(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_WhileLoopStack.empty()){
		CComBSTR Description;
		Description.LoadString(SYNTAX_MATCHING_ENDWHILE_NOT_FOUND);
		Fire_AsyncronBrickError(SYNTAX_MATCHING_ENDWHILE_NOT_FOUND, Description);
		return S_OK;
	}
	int exitAddress = m_WhileLoopStack.top();
	m_WhileLoopStack.pop();
	int startAddress = m_WhileLoopStack.top();
	m_WhileLoopStack.pop();
	int distance;

	// Calculate the distance to the start of the loop.

	distance = (m_pProgramBuffer->GetLength()+1) - startAddress;

	RCX_Command cmd(pbLJump, 0x80|LO_ADDBYTE(distance), HI_ADDBYTE(distance));

	m_pProgramBuffer->AddData(cmd);

	// Calculate the distance to the end of the loop.

	distance = (m_pProgramBuffer->GetLength()) - exitAddress;
	// Get us a pointer to the program data
	BYTE *pBuffer = m_pProgramBuffer->GetData();
	// fill in the address of the end of the loop
	pBuffer[exitAddress] = LO_ADDBYTE(distance);
	pBuffer[exitAddress+1] = HI_ADDBYTE(distance);

	m_pProgramBuffer->DebugDump();
	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::If
// Description	    : Starts building an if/else/endif construct
// Return type		: STDMETHODIMP 
// Argument         : RCX_SOURCE Source1
// Argument         : short Number1
// Argument         : short RelOp
// Argument         : RCX_SOURCE Source2
// Argument         : short Number2
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::If(RCX_SOURCE Source1, short Number1, short RelOp, RCX_SOURCE Source2, short Number2, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	if(!m_pProgramBuffer)
		return S_OK;

	RCX_Command cmd(pbLCheckDo, (BYTE)Source1 | (RelOp<<6), (BYTE)Source2, LO_BYTE(Number1), HI_BYTE(Number1), (BYTE)Number2);

	m_pProgramBuffer->AddData(cmd);

	BYTE pBuff[5];
	pBuff[0] = 1;										// Add two bytes for the addres we fill in later.
	pBuff[1] = (BYTE)0xFF;

	m_IfElseStack.push(m_pProgramBuffer->GetLength());	// Push the address of the address to end the loop
	m_pProgramBuffer->AddData(cmd);
	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::Else
// Description	    : Handle the else part
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::Else(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_IfElseStack.empty()){
		CComBSTR Description;
		Description.LoadString(SYNTAX_MATCHING_ENDIF_NOT_FOUND);
		Fire_AsyncronBrickError(SYNTAX_MATCHING_ENDIF_NOT_FOUND, Description);
		return S_OK;
	}

	int exitAddress = m_IfElseStack.top();
	m_IfElseStack.pop();

	m_IfElseStack.push(m_pProgramBuffer->GetLength()+1);	// Push the address of the address to end the if

	RCX_Command cmd(pbLJump, (BYTE)0, (BYTE)0);

	m_pProgramBuffer->AddData(cmd);

	int distance = (m_pProgramBuffer->GetLength()) - exitAddress;
	// Get us a pointer to the program data
	BYTE *pBuffer = m_pProgramBuffer->GetData();
	// fill in the address of the end of the loop
	pBuffer[exitAddress] = LO_ADDBYTE(distance);
	pBuffer[exitAddress+1] = HI_ADDBYTE(distance);

	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::EndIf
// Description	    : Handles the endif
// Return type		: STDMETHODIMP 
// Argument         : VARIANT_BOOL *ReturnValue
STDMETHODIMP CPhantomCtrl::EndIf(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	

	if(!m_pProgramBuffer)
		return S_OK;

	if(m_IfElseStack.empty()){
		CComBSTR Description;
		Description.LoadString(SYNTAX_MATCHING_ENDIF_NOT_FOUND);
		Fire_AsyncronBrickError(SYNTAX_MATCHING_ENDIF_NOT_FOUND, Description);
		return S_OK;
	}

	int exitAddress = m_IfElseStack.top();
	m_IfElseStack.pop();

	int distance = (m_pProgramBuffer->GetLength()) - exitAddress;
	// Get us a pointer to the program data
	BYTE *pBuffer = m_pProgramBuffer->GetData();
	// fill in the address of the end of the loop
	pBuffer[exitAddress] = LO_ADDBYTE(distance);
	pBuffer[exitAddress+1] = HI_ADDBYTE(distance);

	m_pProgramBuffer->DebugDump();
	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}


// Function name	: CPhantomCtrl::AboutBox
// Description	    : Displays the AboutBox
// Return type		: STDMETHODIMP 
STDMETHODIMP CPhantomCtrl::AboutBox()
{
    CAboutDlg dlg;
    dlg.DoModal();
    return S_OK;
}


// Function name	: CPhantomCtrl::ReportError
// Description	    : Displays a system error in a message box.
// Return type		: void 
// Argument         : int ErrorNum
// Argument         : DWORD SystemError
void CPhantomCtrl::ReportError(int ErrorNum, DWORD SystemError)
{
	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS, 
		NULL, 
		SystemError, 
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf, 
		0, 
		NULL 
	);

	// ReportError(ErrorNum, (LPSTR)lpMsgBuf);
	// Free the buffer.
	LocalFree( lpMsgBuf );
}


// Function name	: CPhantomCtrl::ReportError
// Description	    : Passes an error to the VB program
// Return type		: void 
// Argument         : int ErrorNum
// Argument         : LPSTR ErrorMessage
void CPhantomCtrl::ReportError(int ErrorNum, LPSTR ErrorMessage)
{
	CComBSTR Description;
	Description.LoadString(ErrorNum);
	Fire_AsyncronBrickError(ErrorNum, Description);
}


// Function name	: CPhantomCtrl::CommClose
// Description	    : Closes the com port and kills all the timers.
// Return type		: void 
void CPhantomCtrl::CommClose()
{
	if(m_MoniTimerID){
		m_pMessageSink->KillTimer(m_MoniTimerID);
		m_MoniTimerID = 0;
	}


	int i;
	for(i = 0; i < MAXEVENTS; i++){
		if(m_Target[i].m_TimerID){
			m_pMessageSink->KillTimer(m_Target[i].m_TimerID);
			m_Target[i].m_TimerID = 0;
		}
	}

	if(m_hCom != INVALID_HANDLE_VALUE){
		CloseHandle(m_hCom);
		WaitForSingleObject(m_hReaderThread, INFINITE);
		CloseHandle(m_WriteOverlap.hEvent);
		m_hCom = INVALID_HANDLE_VALUE;
	}
}


// Function name	: CPhantomCtrl::Reader
// Description	    : Reader thread for the comm port
// Return type		: LONG 
LONG CPhantomCtrl::Reader()
{
	BOOL fSuccess;
	DWORD BytesRead = 0;
	BYTE bRecvBuff[MAXBUFF];
	OVERLAPPED ReadOverlap;

	ReadOverlap.hEvent = CreateEvent(
		NULL, // no security attributes 
		FALSE, // auto reset event 
		FALSE, // not signaled 
		NULL    // no name 
		);
	ReadOverlap.Offset = 0;
	ReadOverlap.OffsetHigh = 0;

	for(;;){

		memset(bRecvBuff, 0, MAXBUFF); // temp

		fSuccess = ReadFile(m_hCom, bRecvBuff, MAXBUFF-1, &BytesRead, &ReadOverlap);
		if(!fSuccess){
			if(GetLastError() == ERROR_IO_PENDING){
				fSuccess = GetOverlappedResult(m_hCom, &ReadOverlap, &BytesRead, true);
			}
			else if(GetLastError() == ERROR_INVALID_HANDLE){
				return 0;
			}
			else {
				ReportError(2000, GetLastError());
				return 0;
			}
		}

		if(BytesRead == 0){
			continue;
		}
		m_RecvLen = 0;

		if(IsEcho(bRecvBuff)){
				m_bEchoEvent = true;
			}
		for(unsigned int i = ((m_ComPortNo == USB)?0:m_SendLen) ; i < BytesRead; i++){
			Process_TowerData(bRecvBuff[i]);
		}
	}
	return 0;
}


// Function name	: CPhantomCtrl::Process_TowerData
// Description	    : State machine that decodes responces from the tower and RCX
// Return type		: void 
// Argument         : BYTE inbyte
void CPhantomCtrl::Process_TowerData(BYTE inbyte)
{
	static enum {
		LOOKINGFOR55 = 0, 
		LOOKINGFORFF = 1, 
		LOOKINGFOR00 = 2, 
		LOOKINGFORDATA1 = 3, 
		LOOKINGFORDATA2 = 4, 
		LOOKINGFORCKSUM1 = 5, 
		LOOKINGFORCKSUM2 = 6
	} State = LOOKINGFOR55;

	static BYTE sbRecvBuff[MAXBUFF];
	static int iNextPos=0;
	static int chksum = 0;

	// Place the received byte in the buffer
	sbRecvBuff[iNextPos] = inbyte;
	sbRecvBuff[iNextPos+1] = 0;


	switch(State){
	case LOOKINGFOR55:								// Look for start of the data
		chksum = 0;
		if(sbRecvBuff[iNextPos] == 0x55){
			iNextPos++;
			State = LOOKINGFORFF;
		}
		else {
			iNextPos=0;
			State = LOOKINGFOR55;
		}
		break;
	case LOOKINGFORFF:								// Look for the second byte
		if(sbRecvBuff[iNextPos] == 0xFF){
			iNextPos++;
			State = LOOKINGFOR00;
		}
		else {
			iNextPos=0;
			State = LOOKINGFOR55;
		}
		break;
	case LOOKINGFOR00:								// And now the third
		if(sbRecvBuff[iNextPos] == 0x00){
			iNextPos++;
			State = LOOKINGFORDATA1;
		}
		else {
			iNextPos=0;
			State = LOOKINGFOR55;
		}
		break;
	case LOOKINGFORDATA1:
		chksum = (chksum + sbRecvBuff[iNextPos++]) & 0xff;		// A data byte, add it to the checksum
		State = LOOKINGFORDATA2;
		break;
	case LOOKINGFORDATA2:
		if(sbRecvBuff[iNextPos] == ((~sbRecvBuff[iNextPos-1]) & 0xff)){	// Check that the previous byte was ok.
			iNextPos++;
			State = LOOKINGFORCKSUM1;
		}
		else {
			iNextPos=0;
			State = LOOKINGFOR55;
		}
		break;
	case LOOKINGFORCKSUM1:
		if(chksum == sbRecvBuff[iNextPos]){			// if this byte matches the checksum we are almost at
			iNextPos++;									// the end of the packet
			State = LOOKINGFORCKSUM2;
		}
		else {											// otherwise, treat it as a data byte.
			chksum = (chksum + sbRecvBuff[iNextPos++]) & 0xff;
			State = LOOKINGFORDATA2;
		}
		break;
	case LOOKINGFORCKSUM2:								// if this matches the prev byte, we are done.
		if(sbRecvBuff[iNextPos] == ((~sbRecvBuff[iNextPos-1]) & 0xff)){


			m_RecvLen = iNextPos;

			if(IsReply(sbRecvBuff)){	// check that it is the reply and not the echo by the tower.
				// at this point we have a reply from the RCX, 
				// so we are going to copy it to the receive buffer 
				memcpy(m_bRecvBuff, sbRecvBuff, m_RecvLen+1);
				// and fire the read event so the main thread knows and reply has come back. 
				if (! ::SetEvent(m_hReadEvent) ){	// tell whoever is waiting that their data has arived.
					ReportError(2000, GetLastError());
					return;
				} 
			}
			else {
				if(sbRecvBuff[3] == pbInternMessage){
					m_pMessageSink->SendMessage(PBR_MESSAGE, sbRecvBuff[5]);
				}
			}
		}
		//Time to start looking for a new frame.
		iNextPos=0;
		State = LOOKINGFOR55;
		memset(sbRecvBuff, 0, MAXBUFF); // temp
		chksum = 0;
		break;
	}
}


// Function name	: CPhantomCtrl::IsEcho
// Description	    : Check to see if the read data is the echo from the tower
// Return type		: bool 
// Argument         : BYTE *RecvBuff
bool CPhantomCtrl::IsEcho(BYTE *RecvBuff)
{
	if(memcmp(RecvBuff, m_bSendBuff, m_SendLen)==0){
		return true;
	}
	return false;
}


// Function name	: CPhantomCtrl::IsReply
// Description	    : Check if the read data is a reply to the last command sent.
// Return type		: bool 
// Argument         : BYTE *RecvBuff
bool CPhantomCtrl::IsReply(BYTE *RecvBuff)
{
	if(RecvBuff[3] == ((~m_bSendBuff[3]) & 0xff)){
		return true;
	}
	return false;
}


// Function name	: CPhantomCtrl::LoadOLEString
// Description	    : load a BSTR from the resources.
// Return type		: LPOLESTR 
// Argument         : UINT uID
LPOLESTR CPhantomCtrl::LoadOLEString(UINT uID)
{
	LPOLESTR str;
	TCHAR sz[512];
	UINT nLen = ::LoadString(_Module.GetResourceInstance(), uID, sz, 512);
	ATLASSERT(nLen < 511);
	nLen++;
	str = (LPOLESTR)CoTaskMemAlloc(nLen * sizeof(OLECHAR));
	MultiByteToWideChar(CP_ACP, 0, sz, -1, str, nLen);
	return str;
}



// Function name	: CPhantomCtrl::SendData
// Description	    : Send the buffer to the RCX and wait for a reply.
// Return type		: void 
// Argument         : BYTE *InBuff
// Argument         : BYTE *OutBuff
// Argument         : DWORD *BytesRead
void CPhantomCtrl::SendData(BYTE *InBuff, BYTE *OutBuff, DWORD *BytesRead)
{
	int retval = 0;

	BOOL fSuccess;

	DWORD BytesWritten;
	int CheckSum = 0;

	m_SendLen = 0;
	memset(m_bSendBuff, 0, MAXBUFF);
	DWORD timeout = CalcTimeOut(*InBuff);

	int InBuffLen = (*InBuff &0x07);	//Get number of args

	if(InBuffLen > 5) InBuffLen -=6;	//Six and Seven mean 0 and 1

	if(*InBuff == pbContinueDownLoad){	// For this the length is 5 + the value in bytes 3 and 4
		InBuffLen = RCX_VALUE(InBuff[4], InBuff[3])+5;
	}

	InBuffLen++;						// Plus one for the command itsself

	m_bSendBuff[m_SendLen++] = 0x55;	// start with the sync bytes
    m_bSendBuff[m_SendLen++] = 0xff;
    m_bSendBuff[m_SendLen++] = 0x00;
	*InBuff |= m_RetryBit;				// each new command has the retry bit inverted.
	m_RetryBit ^= pbRetryBit;

    while (InBuffLen--) {				
		m_bSendBuff[m_SendLen++] = *InBuff;				// copy all the data and
		m_bSendBuff[m_SendLen++] = (~*InBuff) & 0xff;	// its inverse
		CheckSum += *InBuff++;							// calculate its checksum
	}
    m_bSendBuff[m_SendLen++] = CheckSum;				// add the checksum
    m_bSendBuff[m_SendLen++] = ~CheckSum;				// and its invers

	m_bEchoEvent = false;		// this turns true when the tower echos back the command			

	short RetryCnt = 0;			// Set the retry cound

	if(m_hCom == INVALID_HANDLE_VALUE){
		::MessageBox(m_pMessageSink->m_hWnd, "Remember to call InitCom() first.", "Phantom Error", MB_OK | MB_ICONINFORMATION );
		return;
	}

	// Start sending the data.
	while(RetryCnt < m_ImmidiateRetriesLimit){

		// Send the data.
		fSuccess = WriteFile(m_hCom, m_bSendBuff, (DWORD)m_SendLen, &BytesWritten, &m_WriteOverlap);
		if(GetLastError() == ERROR_IO_PENDING){
			// wait for the i/o to complete.
			fSuccess = GetOverlappedResult(m_hCom, &m_WriteOverlap, &BytesWritten, true);
			// oops, something is wrong.
			if (!fSuccess) {
				ReportError(1000, GetLastError());
				return;
			}
		}

		// Wait for the reply 
		*OutBuff = 0;
		*BytesRead = 0;
		DWORD dwWaitResult = WaitForSingleObject(m_hReadEvent, timeout);
		if(dwWaitResult == WAIT_OBJECT_0){
			RetryCnt = 0;
			// We have received a reply, so copy it to the outbuff
			for(int x = 3; x < (m_RecvLen-2); x+=2){
				if(x==3)	// remove the retry bit.
					*OutBuff = m_bRecvBuff[x] | pbRetryBit;
				else
					*OutBuff = m_bRecvBuff[x];
				OutBuff++;
				*OutBuff = 0;
				(*BytesRead)++;
			}
			// Record a success and return
			m_ShortTermRetryCtr.Success(RetryCnt);
			m_LongTermRetryCtr.Success(RetryCnt);
			return;
		}
		// No reply, so lets send again.
		RetryCnt++;
	}
	// Too Many Retrys.
	CComBSTR Description;
	Description.LoadString(TOO_MANY_RESENDS);
	Fire_AsyncronBrickError(TOO_MANY_RESENDS, Description);
	// Record a failure.
	m_ShortTermRetryCtr.Failure();
	m_LongTermRetryCtr.Failure();
}


// Function name	: CPhantomCtrl::SendDataNoReply
// Description	    : 
// Return type		: bool 
// Argument         : BYTE *InBuff
bool CPhantomCtrl::SendDataNoReply(BYTE *InBuff)
{
	int retval = 0;

	BOOL fSuccess;

	DWORD BytesWritten;
	int CheckSum = 0;

	m_SendLen = 0;

	int InBuffLen = (*InBuff &0x07);	//Get number of args

	if(InBuffLen > 5) InBuffLen -=6;	//Six and Seven mean 0 and 1

	InBuffLen++;						// Plus one for the command itsself

	m_bSendBuff[m_SendLen++] = 0x55;
    m_bSendBuff[m_SendLen++] = 0xff;
    m_bSendBuff[m_SendLen++] = 0x00;
	*InBuff |= m_RetryBit;
	m_RetryBit ^= pbRetryBit;

    while (InBuffLen--) {
		m_bSendBuff[m_SendLen++] = *InBuff;
		m_bSendBuff[m_SendLen++] = (~*InBuff) & 0xff;
		CheckSum += *InBuff++;
	}
    m_bSendBuff[m_SendLen++] = CheckSum;
    m_bSendBuff[m_SendLen++] = ~CheckSum;

	m_bEchoEvent = false;

	if(m_hCom == INVALID_HANDLE_VALUE){
		::MessageBox(m_pMessageSink->m_hWnd, "Remember to call InitCom() first.", "Phantom Error", MB_OK | MB_ICONINFORMATION );
		return false;
	}

	fSuccess = WriteFile(m_hCom, m_bSendBuff, (DWORD)m_SendLen, &BytesWritten, &m_WriteOverlap);
	if(GetLastError() == ERROR_IO_PENDING){
		fSuccess = GetOverlappedResult(m_hCom, &m_WriteOverlap, &BytesWritten, true);

		if (!fSuccess) {
			ReportError(1000, GetLastError());
			return false;
		}
	}
	return true;
}


// Function name	: CPhantomCtrl::CalcTimeOut
// Description	    : Some commands take longer for the rcx to reply
// Return type		: DWORD 
// Argument         : BYTE command
DWORD CPhantomCtrl::CalcTimeOut(BYTE command)
{
	if((command == pbMemMap) || 
		(command == pbUpload)  || 
		(command == pbContinueDownLoad )|| 
		(command == pbGoIntoBootMode ) ||
		(command == pbUnlockFirmware))
		return 2000;
	else
		return 300;

}


// Function name	: CPhantomCtrl::ContinueFirmwareDownLoad
// Description	    : Send the next block of code to the RCX
// Return type		: bool 
// Argument         : int BlockNumber
// Argument         : int Length
// Argument         : BYTE *buffer
// Argument         : BYTE *retval
bool CPhantomCtrl::ContinueFirmwareDownLoad(int BlockNumber, int Length, BYTE *buffer, BYTE *retval)
{
	int cksum = 0;
	BYTE pBuff[BUFFERSIZE];
	BYTE pReply[5];
	DWORD ReplyLeng;
	pBuff[0] = pbContinueDownLoad;
	pBuff[1] = LO_BYTE(BlockNumber);
	pBuff[2] = HI_BYTE(BlockNumber);
	pBuff[3] = LO_BYTE(Length);
	pBuff[4] = HI_BYTE(Length);
	memcpy(&pBuff[5], buffer, Length);
	for (int i = 0; i < Length; i++)
		cksum += buffer[i];
	pBuff[Length + 5] = LO_BYTE(cksum);

	SendData(pBuff, pReply, &ReplyLeng);
	if(ReplyLeng == 2 && pReply[0] == RCX_REPLY(pbContinueDownLoad)){
		*retval = pReply[1];
		switch(pReply[1]){
		case 0:
			return TRUE;
			break;
		case 3:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR_IN_DOWNLOAD_CHECKSUM, (LPARAM)0);
			break;
		case 4:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR_IN_DOWNLOAD_RAMCHECKSUMERROR, (LPARAM)0);
			break;
		case 6:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_BRICK_IS_NOT_IN_DL_MODE, (LPARAM)0);
			break;
		default:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR_UNKNOWN, (LPARAM)0);
		}
	}
	return FALSE;
}


// Function name	: CPhantomCtrl::GoIntoBootMode
// Description	    : Boot the RCX
// Return type		: bool 
bool CPhantomCtrl::GoIntoBootMode()
{

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbGoIntoBootMode, 1, 3, 5, 7, 11);

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbGoIntoBootMode))
		return TRUE;
	return FALSE;
}


// Function name	: CPhantomCtrl::BeginFirmwareDownLoad
// Description	    : Start a Firmware Download
// Return type		: bool 
// Argument         : int StartAddress
// Argument         : int Checksum
bool CPhantomCtrl::BeginFirmwareDownLoad(int StartAddress, int Checksum)
{

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbBeginFirmwareDownLoad, LO_BYTE(StartAddress), HI_BYTE(StartAddress), LO_BYTE(Checksum), HI_BYTE(Checksum), 0);

	SendData(cmd, pReply, &ReplyLeng);
	if(ReplyLeng == 2 && pReply[0] == RCX_REPLY(pbBeginFirmwareDownLoad))
		return TRUE;
	return FALSE;
}


// Function name	: CPhantomCtrl::ParseMotorList
// Description	    : Convert the motor string to bits.
// Return type		: bool 
// Argument         : BSTR MotorList
// Argument         : BYTE *motors
bool CPhantomCtrl::ParseMotorList(BSTR MotorList, BYTE *motors)
{
	CComBSTR foo;
	foo.Attach(MotorList);
	*motors = 0;

	for(unsigned int i = 0; i < foo.Length(); i++){
		if(foo[i] == '0')
			*motors |= 0x01;
		else if(foo[i] == '1')
			*motors |= 0x02;
		else if(foo[i] == '2')
			*motors |= 0x04;
	}

	foo.Detach();
	if(*motors == 0)
		return false;

	return true;
}


// Function name	: CPhantomCtrl::PollPort
// Description	    : Read a value from the RCX and fire the correct event.
// Return type		: void 
// Argument         : int IDEvent
void CPhantomCtrl::PollPort(int IDEvent)
{
	int PollIndex = IDEvent-TIMEROFFSET;
	short lastval = m_Target[PollIndex].m_LastVal;

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPoll, BYTE(m_Target[PollIndex].m_Source), BYTE(m_Target[PollIndex].m_Number));

	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPoll)){
		m_Target[PollIndex].m_LastVal = RCX_VALUE(pReply[2], pReply[1]);
		if(lastval != m_Target[PollIndex].m_LastVal){
			if(m_Target[PollIndex].m_Source == SRC_VAR)
				Fire_VariableChange(m_Target[PollIndex].m_Number, m_Target[PollIndex].m_LastVal);
			if((m_Target[PollIndex].m_Source == SRC_SENVAL) || 
				(m_Target[PollIndex].m_Source == SRC_SENRAW) || 
				(m_Target[PollIndex].m_Source == SRC_SENBOOL))
				Fire_InputChange(m_Target[PollIndex].m_Number, m_Target[PollIndex].m_LastVal);

		}
	}
}


// Function name	: CPhantomCtrl::CheckConnection
// Description	    : Make sure that InitCom has been called.
// Return type		: bool 
bool CPhantomCtrl::CheckConnection()
{
	if(m_hCom == INVALID_HANDLE_VALUE){
		::MessageBox(m_pMessageSink->m_hWnd, "Remember to call InitCom() first.", "Phantom Error", MB_OK | MB_ICONINFORMATION );
		return false;
	}
	return true;
}


// Function name	: CPhantomCtrl::SetRetransmitRetries
// Description	    : Set the retry limits
// Return type		: STDMETHODIMP 
// Argument         : short immidiateRetries
// Argument         : short downloadRetries
// Argument         : VARIANT_BOOL *retval
STDMETHODIMP CPhantomCtrl::SetRetransmitRetries(short immidiateRetries, short downloadRetries, VARIANT_BOOL *retval)
{
	*retval = VARIANT_TRUE;
	m_ImmidiateRetriesLimit = immidiateRetries;
	m_DownloadRetriesLimit = downloadRetries;

	return S_OK;
}



// Function name	: CPhantomCtrl::BeginSubDownload
// Description	    : Start the download of a sub
// Return type		: bool 
// Argument         : int SubNumber
// Argument         : int Size
bool CPhantomCtrl::BeginSubDownload(int SubNumber, int Size)
{

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbBeginOfSub, 0, SubNumber, 0, LO_BYTE(Size), HI_BYTE(Size));

	SendData(cmd, pReply, &ReplyLeng);
	if(ReplyLeng == 2 && pReply[0] == RCX_REPLY(pbBeginOfSub)){
		CComBSTR Description;
		switch(pReply[1]){
		case 0:
			break;
		case 1:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_NOT_ENOUGH_MEMORY, (LPARAM)0);
			return FALSE;
			break;
		case 2:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR, (LPARAM)0);
			return FALSE;
			break;
		default:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR_UNKNOWN, (LPARAM)0);
			return FALSE;
		}
		return TRUE;
	}
	return FALSE;
}


// Function name	: CPhantomCtrl::BeginTaskDownload
// Description	    : Start the download of a Task
// Return type		: bool 
// Argument         : int TaskNumber
// Argument         : int Size
bool CPhantomCtrl::BeginTaskDownload(int TaskNumber, int Size)
{

	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbBeginOfTask, 0, TaskNumber, 0, LO_BYTE(Size), HI_BYTE(Size));

	SendData(cmd, pReply, &ReplyLeng);
	if(ReplyLeng == 2 && pReply[0] == RCX_REPLY(pbBeginOfTask)){
		switch(pReply[1]){
		case 0:
			break;
		case 1:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_NOT_ENOUGH_MEMORY, (LPARAM)0);
			return FALSE;
			break;
		case 2:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR, (LPARAM)0);
			return FALSE;
			break;
		default:
			m_pMessageSink->SendMessage(PBR_ERROR , DOWNLOAD_ERROR_UNKNOWN, (LPARAM)0);
			return FALSE;
		}
		return TRUE;
	}
	return FALSE;
}

STDMETHODIMP CPhantomCtrl::KeepTowerAlive(VARIANT_BOOL state)
{
	if(m_ComPortNo != USB){
		if(state == VARIANT_TRUE){
			if(m_MoniTimerID == 0){
				m_MoniTimerID = m_pMessageSink->SetTimer(MONITIMER, 4000);
			}
		}
		else{
			if(m_MoniTimerID != 0){
				m_pMessageSink->KillTimer(m_MoniTimerID);
				m_MoniTimerID = 0;
			}
		}
	}

	return S_OK;
}

void CPhantomCtrl::OnMonitor(int Param)
{
	DWORD BytesWritten;
	BOOL fSuccess;
	char *pp = "\x13";

	fSuccess = WriteFile(m_hCom, pp, (DWORD)1, &BytesWritten, &m_WriteOverlap);
	if(GetLastError() == ERROR_IO_PENDING){
		// wait for the i/o to complete.
		fSuccess = GetOverlappedResult(m_hCom, &m_WriteOverlap, &BytesWritten, true);
		// oops, something is wrong.
		if (!fSuccess) {
			ReportError(1000, GetLastError());
			return;
		}
	}
}

STDMETHODIMP CPhantomCtrl::ExitSub(VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	



	RCX_Command cmd(pbEndOfSub);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
	}
	*ReturnValue = VARIANT_TRUE;
	return S_OK;
}

void CPhantomCtrl::SendData(RCX_Command &cmd, BYTE *OutBuff, DWORD *BytesRead)

{
	SendData(cmd.GetBody(), OutBuff, BytesRead);
}

STDMETHODIMP CPhantomCtrl::PlayToneV(short Number, short Time, VARIANT_BOOL *ReturnValue)
{
	*ReturnValue = VARIANT_FALSE;
	if(CheckConnection()==false)
		return S_OK;	


	BYTE pReply[5];
	DWORD ReplyLeng;

	RCX_Command cmd(pbPlayToneVar, (BYTE)Number, (BYTE)Time);

	if(m_pProgramBuffer){
		m_pProgramBuffer->AddData(cmd);
		*ReturnValue = VARIANT_TRUE;
		return S_OK;
	}
	SendData(cmd, pReply, &ReplyLeng);
	if(pReply[0] == RCX_REPLY(pbPlayToneVar))
		*ReturnValue = VARIANT_TRUE;

	return S_OK;
}


