// PhantomCtrl.h : Declaration of the CPhantomCtrl
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is PhantomCtrl.h.
 *
 * The Initial Developer of the Original Code is
 * Fenestra Software LLC.
 * Portions created by the Initial Developer are Copyright (C) 2002 Fenestra Software LLC
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#ifndef __PHANTOMCTRL_H_
#define __PHANTOMCTRL_H_

#include "resource.h"       // main symbols
#include <atlctl.h>
#include <stack>
#include "PhantomCP.h"
#include "MessageSink.h"	// Added by ClassView
#include "FirmwareData.h"
#include "PollTarget.h"
#include "RetryCounter.h"	// Added by ClassView
#include "ProgramData.h"

using namespace std;
typedef stack<int> STACK_INT;


const int TRANSFER_SIZE = 200;
const int USB_TRANSFER_SIZE = 100;
const int PROGRAM_TRANSFER_SIZE = 20;

const int BUFFERSIZE = 4096;
const int MAXBUFF = 1024;
const int MAXEVENTS = 35;

const BYTE pbRetryBit = 0x08;

/////////////////////////////////////////////////////////////////////////////
// CPhantomCtrl
class ATL_NO_VTABLE CPhantomCtrl : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDispatchImpl<IPhantomCtrl, &IID_IPhantomCtrl, &LIBID_PHANTOMLib>,
	public CComControl<CPhantomCtrl>,
	public IPersistStreamInitImpl<CPhantomCtrl>,
	public IOleControlImpl<CPhantomCtrl>,
	public IOleObjectImpl<CPhantomCtrl>,
	public IOleInPlaceActiveObjectImpl<CPhantomCtrl>,
	public IViewObjectExImpl<CPhantomCtrl>,
	public IOleInPlaceObjectWindowlessImpl<CPhantomCtrl>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CPhantomCtrl>,
	public IPersistStorageImpl<CPhantomCtrl>,
	public IPersistPropertyBagImpl<CPhantomCtrl>,
	public ISpecifyPropertyPagesImpl<CPhantomCtrl>,
	public IQuickActivateImpl<CPhantomCtrl>,
	public IDataObjectImpl<CPhantomCtrl>,
	public IProvideClassInfo2Impl<&CLSID_PhantomCtrl, &DIID__IPhantomCtrlEvents, &LIBID_PHANTOMLib>,
	public IPropertyNotifySinkCP<CPhantomCtrl>,
	public CComCoClass<CPhantomCtrl, &CLSID_PhantomCtrl>,
	public IPerPropertyBrowsingImpl<CPhantomCtrl>,
	public CProxy_IPhantomCtrlEvents< CPhantomCtrl >
{
public:
	CPhantomCtrl()
	{
		m_pMessageSink = new CMessageSink(this);
		m_pFirmwareData = NULL;

		//Load bitmap for icon.
		BITMAP bmp;	

		m_hBitmap = LoadBitmap(_Module.GetResourceInstance(),  MAKEINTRESOURCE(IDB_PHANTOMCTRL2));
		GetObject(m_hBitmap, sizeof(BITMAP), (LPSTR)&bmp);
		m_bmSize.cx = bmp.bmWidth;
		m_bmSize.cy = bmp.bmHeight;

		AtlPixelToHiMetric( &m_bmSize, &m_sizeExtent);
		m_sizeNatural = m_sizeExtent;
		m_bAutoSize = true;

		//Set default values
		m_DownloadRetriesLimit = 10;
		m_ImmidiateRetriesLimit = 5;

		// Initalize vars
		m_hCom = INVALID_HANDLE_VALUE;
		m_ComPortNo = USB;
		m_LinkType = INFRARED;
		m_Brick = RCX2;
		m_RetryBit = 0;
		memset(m_bSendBuff,0,MAXBUFF);
		m_SendLen = 0;
		memset(m_bRecvBuff,0,MAXBUFF);
		m_RecvLen = 0;


		//Load the commport strings.

		LoadString(_Module.GetResourceInstance(),IDS_NAM_COMPORT0,m_CommPortStr[0],255);
		LoadString(_Module.GetResourceInstance(),IDS_NAM_COMPORT1,m_CommPortStr[1],255);
		LoadString(_Module.GetResourceInstance(),IDS_NAM_COMPORT2,m_CommPortStr[2],255);
		LoadString(_Module.GetResourceInstance(),IDS_NAM_COMPORT3,m_CommPortStr[3],255);
		LoadString(_Module.GetResourceInstance(),IDS_NAM_COMPORT4,m_CommPortStr[4],255);
		m_pProgramBuffer = NULL;
		m_MoniTimerID = 0;

	}
	~CPhantomCtrl()
	{
		CommClose();
		if(m_pFirmwareData){
			delete m_pFirmwareData;
			m_pFirmwareData = NULL;
		}

		m_pMessageSink->DestroyWindow();
		delete m_pMessageSink;
		m_pMessageSink = NULL;
		if(m_pProgramBuffer){
			delete m_pProgramBuffer;
			m_pProgramBuffer = NULL;
		}
	}

DECLARE_REGISTRY_RESOURCEID(IDR_PHANTOMCTRL)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CPhantomCtrl)
	COM_INTERFACE_ENTRY(IPhantomCtrl)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(IPersistPropertyBag)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
	COM_INTERFACE_ENTRY(IPerPropertyBrowsing)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

BEGIN_PROP_MAP(CPhantomCtrl)
//	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
//	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
	PROP_ENTRY("ComPortNo",1,CLSID_NULL)
	PROP_ENTRY("LinkType",2,CLSID_NULL)
	PROP_ENTRY("Brick",3,CLSID_NULL)
END_PROP_MAP()

BEGIN_CONNECTION_POINT_MAP(CPhantomCtrl)
	CONNECTION_POINT_ENTRY(IID_IPropertyNotifySink)
	CONNECTION_POINT_ENTRY(DIID__IPhantomCtrlEvents)
END_CONNECTION_POINT_MAP()

BEGIN_MSG_MAP(CPhantomCtrl)
	CHAIN_MSG_MAP(CComControl<CPhantomCtrl>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
	{
		static const IID* arr[] = 
		{
			&IID_IPhantomCtrl,
		};
		for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
		{
			if (InlineIsEqualGUID(*arr[i], riid))
				return S_OK;
		}
		return S_FALSE;
	}

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// Get the predefined porperty strings.
	STDMETHOD(GetPredefinedStrings)(DISPID dispID, CALPOLESTR *pCaStringsOut,CADWORD *pCaCookiesOut)
	{
		AtlTrace("GetPredefinedStrings dispID=%d\n",dispID);
		if (pCaStringsOut == NULL || pCaCookiesOut == NULL)
			return E_POINTER;
		ULONG cElems = 0;
		switch(dispID){
		case 1:
			pCaStringsOut->cElems = 5;
			pCaStringsOut->pElems = (LPOLESTR*)CoTaskMemAlloc(sizeof(LPOLESTR) * pCaStringsOut->cElems);
			if (pCaStringsOut->pElems == NULL)
				return E_OUTOFMEMORY;

			pCaCookiesOut->cElems = 5;
			pCaCookiesOut->pElems = (DWORD*)CoTaskMemAlloc(sizeof(DWORD*) * pCaCookiesOut->cElems);

			if (pCaCookiesOut->pElems == NULL)
			{
				CoTaskMemFree(pCaStringsOut->pElems);
				return E_OUTOFMEMORY;
			}

			pCaCookiesOut->pElems[0] = USB;
			pCaStringsOut->pElems[0] = LoadOLEString(IDS_COMPORT0);
			pCaCookiesOut->pElems[1] = COM1;
			pCaStringsOut->pElems[1] = LoadOLEString(IDS_COMPORT1);
			pCaCookiesOut->pElems[2] = COM2;
			pCaStringsOut->pElems[2] = LoadOLEString(IDS_COMPORT2);
			pCaCookiesOut->pElems[3] = COM3;
			pCaStringsOut->pElems[3] = LoadOLEString(IDS_COMPORT3);
			pCaCookiesOut->pElems[4] = COM4;
			pCaStringsOut->pElems[4] = LoadOLEString(IDS_COMPORT4);
			return S_OK;
		case 2:
			pCaStringsOut->cElems = 3;
			pCaStringsOut->pElems = (LPOLESTR*)CoTaskMemAlloc(sizeof(LPOLESTR) * pCaStringsOut->cElems);
			if (pCaStringsOut->pElems == NULL)
				return E_OUTOFMEMORY;

			pCaCookiesOut->cElems = 3;
			pCaCookiesOut->pElems = (DWORD*)CoTaskMemAlloc(sizeof(DWORD*) * pCaCookiesOut->cElems);

			if (pCaCookiesOut->pElems == NULL)
			{
				CoTaskMemFree(pCaStringsOut->pElems);
				return E_OUTOFMEMORY;
			}
			pCaCookiesOut->pElems[0] = INFRARED;
			pCaStringsOut->pElems[0] = LoadOLEString(IDS_LINKIRD);
			pCaCookiesOut->pElems[1] = CABLE;
			pCaStringsOut->pElems[1] = LoadOLEString(IDS_LINKCBL);
			pCaCookiesOut->pElems[2] = RADIO;
			pCaStringsOut->pElems[2] = LoadOLEString(IDS_LINKRF);
			return S_OK;
		case 3:
			pCaStringsOut->cElems = 3;
			pCaStringsOut->pElems = (LPOLESTR*)CoTaskMemAlloc(sizeof(LPOLESTR) * pCaStringsOut->cElems);
			if (pCaStringsOut->pElems == NULL)
				return E_OUTOFMEMORY;

			pCaCookiesOut->cElems = 3;
			pCaCookiesOut->pElems = (DWORD*)CoTaskMemAlloc(sizeof(DWORD*) * pCaCookiesOut->cElems);

			if (pCaCookiesOut->pElems == NULL)
			{
				CoTaskMemFree(pCaStringsOut->pElems);
				return E_OUTOFMEMORY;
			}
			pCaCookiesOut->pElems[0] = RCX2;
			pCaStringsOut->pElems[0] = LoadOLEString(IDS_RCX2);
			pCaCookiesOut->pElems[1] = SCOUT;
			pCaStringsOut->pElems[1] = LoadOLEString(IDS_SCOUT);
			pCaCookiesOut->pElems[2] = MICROSCOUT;
			pCaStringsOut->pElems[2] = LoadOLEString(IDS_MICROSCOUT);
			
			return S_OK;
		}
		return IPerPropertyBrowsingImpl<CPhantomCtrl>::GetPredefinedStrings(dispID, pCaStringsOut, pCaCookiesOut);
	}

// Get the predefined porperty values.
	STDMETHOD(GetPredefinedValue)( DISPID dispID, DWORD dwCookie, VARIANT* pVarOut)
	{
		AtlTrace("GetPredefinedValue dispID=%d\n",dispID);
		switch(dispID){
		case 1:
		case 2:
		case 3:
			VariantClear(pVarOut);
			V_VT(pVarOut) = VT_I4;
			V_I4(pVarOut) = dwCookie;
			return S_OK;
		}
		return IPerPropertyBrowsingImpl<CPhantomCtrl>::GetPredefinedValue(dispID, dwCookie, pVarOut);
	}

// Get the predefined porperty display strings
	STDMETHOD(GetDisplayString)(DISPID dispID,BSTR *pBstr)
	{
		AtlTrace("GetDisplayString dispID=%d\n",dispID);
		CComBSTR temp;
		switch(dispID){
		case 1:
			switch(m_ComPortNo){
			case USB:
				temp.LoadString(IDS_COMPORT0);
				break;
			case COM1:
				temp.LoadString(IDS_COMPORT1);
				break;
			case COM2:
				temp.LoadString(IDS_COMPORT2);
				break;
			case COM3:
				temp.LoadString(IDS_COMPORT3);
				break;
			case COM4:
				temp.LoadString(IDS_COMPORT4);
				break;
			}			
			*pBstr=temp.Copy();
			return S_OK;
		case 2:
			switch(m_LinkType){
			case INFRARED:
				temp.LoadString(IDS_LINKIRD);
				break;
			case CABLE:
				temp.LoadString(IDS_LINKCBL);
				break;
			case RADIO:
				temp.LoadString(IDS_LINKRF);
				break;
			}
			*pBstr=temp.Copy();
			return S_OK;
		case 3:
			switch(m_Brick){
			case RCX2:
				temp.LoadString(IDS_RCX2);
				break;
			case SCOUT:
				temp.LoadString(IDS_SCOUT);
				break;
			case MICROSCOUT:
				temp.LoadString(IDS_MICROSCOUT);
				break;
			}
			*pBstr=temp.Copy();
			return S_OK;
		}
		return IPerPropertyBrowsingImpl<CPhantomCtrl>::GetDisplayString(dispID, pBstr);
	}


// IPhantomCtrl
public:
	STDMETHOD(PlayToneV)(/*[in]*/short Number,/*[in]*/short Time,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	void SendData(RCX_Command &cmd, BYTE *OutBuff, DWORD *BytesRead);
	STDMETHOD(ExitSub)(/*[out,retval]*/ VARIANT_BOOL *ReturnValue);
	void OnMonitor(int Param);
	STDMETHOD(KeepTowerAlive)(/*[in]*/ VARIANT_BOOL state);
	bool BeginTaskDownload(int TaskNumber, int Size);
	bool BeginSubDownload(int SubNumber, int Size);
	STDMETHOD(SetRetransmitRetries)(/*[in]*/ short immidiateRetries,/*[in]*/ short downloadRetries, /*[out,retval]*/ VARIANT_BOOL* retval);
	void PollPort(int PollIndex);
	LONG FirmwareDownloader();
	LONG ProgramDownloader();
	LONG Reader();
	void ReportError(int ErrorNum, LPSTR ErrorMessage);
	void ReportError(int ErrorNum, DWORD SystemError);
	STDMETHOD(AboutBox)();
	STDMETHOD(EndIf)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Else)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(If)(/*[in]*/ RCX_SOURCE Source1, /*[in]*/ short Number1, /*[in]*/ short RelOp, /*[in]*/ RCX_SOURCE Source2, /*[in]*/ short Number2, /*[out,retval]*/ VARIANT_BOOL* ReturnValue);
	STDMETHOD(EndWhile)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(While)(/*[in]*/ RCX_SOURCE Source1, /*[in]*/ short Number1, /*[in]*/ short RelOp,/*[in]*/ RCX_SOURCE Source2,/*[in]*/ short Number2, /*[out,retval]*/ VARIANT_BOOL* ReturnValue);
	STDMETHOD(EndLoop)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Loop)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(EndOfSubNoDownload)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(EndOfSub)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(BeginOfSub)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(EndOfTaskNoDownload)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(EndOfTask)(/*[out,retval]*/short *ReturnValue);
	STDMETHOD(BeginOfTask)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(StopTask)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(StopAllTasks)(/*[out, retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(StartTask)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(GoSub)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SelectPrgm)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(DeleteTask)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(DeleteSub)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(DeleteAllTasks)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(DeleteAllSubs)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(UnlockPBrick)(/*[out,retval]*/BSTR *ReturnValue);
	STDMETHOD(UnlockFirmware)(/*[in]*/BSTR UnlockString,/*[out,retval]*/BSTR *ReturnValue);
	STDMETHOD(DownloadFirmware)(/*[in]*/BSTR Filename,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetSensorType)(/*[in]*/short Number,/*[in]*/short Type,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetSensorMode)(/*[in]*/short Number,/*[in]*/short Mode,/*[in]*/short Slope,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(ClearSensorValue)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetWatch)(/*[in]*/short Hours,/*[in]*/short Min,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SendPBMessage)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SelectDisplay)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(PlayTone)(/*[in]*/short Frequency,/*[in]*/short Time,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(PlaySystemSound)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(PBTxPower)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(PBTurnOff)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(PBPowerdownTime)(/*[in]*/short Time,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(ClearTimer)(/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(ClearPBMessage)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetEvent)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[in]*/short Time,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Poll)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/short *ReturnValue);
	STDMETHOD(PBBattery)(/*[out,retval]*/short *ReturnValue);
	STDMETHOD(MemMap)(/*[out,retval]*/VARIANT *ReturnValue);
	STDMETHOD(ClearEvent)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(ClearAllEvents)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(UploadDatalog)(/*[in]*/short From,/*[in]*/short Size,/*[out,retval]*/VARIANT *ReturnValue);
	STDMETHOD(SetDatalog)(/*[in]*/short Size,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(DatalogNext)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SumVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SubVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SgnVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(OrVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(MulVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(DivVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(AndVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(AbsVar)(/*[in]*/short VarNo,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Wait)(/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetRwd)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetPower)(/*[in]*/BSTR MotorList,/*[in]*/RCX_SOURCE Source,/*[in]*/short Number,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(SetFwd)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(OnWaitDifferent)(/*[in]*/ BSTR MotorList,/*[in]*/ short Number0,/*[in]*/ short Number1,/*[in]*/ short Number2, /*[in]*/ short Time, /*[out,retval]*/ VARIANT_BOOL *ReturnValue);
	STDMETHOD(OnWait)(/*[in]*/BSTR MotorList,/*[in]*/short Number,/*[in]*/short Time,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(On)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Off)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Float)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(Drive)(/*[in]*/short Number0,/*[in]*/short Number1,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(ClearTachoCounter)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(AlterDir)(/*[in]*/BSTR MotorList,/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(TowerAndCableConnected)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(TowerAlive)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(PBAliveOrNot)(/*[out,retval]*/VARIANT_BOOL *ReturnValue);
	STDMETHOD(GetLongTermRetransmitStatistics)(/*[out,retval]*/ VARIANT *ReturnValue);
	STDMETHOD(GetShortTermRetransStatistics)(/*[out,retval]*/ VARIANT *ReturnValue);
	STDMETHOD(IgnDLerrUntilGoodAnswer)(/*[out,retval]*/ VARIANT_BOOL *ReturnValue);
	STDMETHOD(CloseComm)(/*[out,retval]*/ VARIANT_BOOL *ReturnValue);
	STDMETHOD(InitComm)(/*[out,retval]*/ VARIANT_BOOL *ReturnValue);
	STDMETHOD(get_Brick)(/*[out, retval]*/ PBRICKOPTIONS *pVal);
	STDMETHOD(put_Brick)(/*[in]*/ PBRICKOPTIONS newVal);
	STDMETHOD(get_LinkType)(/*[out, retval]*/ LINKTYPEOPTIONS *pVal);
	STDMETHOD(put_LinkType)(/*[in]*/ LINKTYPEOPTIONS newVal);
	STDMETHOD(get_ComPortNo)(/*[out, retval]*/ COMPORTOPTIONS *pVal);
	STDMETHOD(put_ComPortNo)(/*[in]*/ COMPORTOPTIONS newVal);

	HRESULT OnDraw(ATL_DRAWINFO& di)
	{
		// Just draw the icon
		RECT& rc = *(RECT*)di.prcBounds;
		HDC hdcMem = CreateCompatibleDC(di.hdcDraw);
		if(!hdcMem)
			return S_OK;

		SelectObject(hdcMem, m_hBitmap);
		BitBlt(di.hdcDraw, rc.left, rc.top, m_bmSize.cx, m_bmSize.cy, hdcMem, 0, 0, SRCCOPY);
		DeleteDC(hdcMem);
		return S_OK;
	}
private:
	STACK_INT m_IfElseStack;
	STACK_INT m_WhileLoopStack;
	ProgramData *m_pProgramBuffer;
	bool CheckConnection();
	bool ParseMotorList(BSTR MotorList, BYTE *motors);
	bool BeginFirmwareDownLoad(int StartAddress, int Checksum);
	bool GoIntoBootMode();
	bool ContinueFirmwareDownLoad(int BlockNumber, int Length, BYTE *buffer, BYTE *retval);
	LPOLESTR LoadOLEString(UINT uID);
	DWORD CalcTimeOut(BYTE command);
	bool SendDataNoReply(BYTE *InBuff);
	void SendData(BYTE *InBuff, BYTE *OutBuff, DWORD *BytesRead);
	bool IsReply(BYTE *RecvBuff);
	void GetRetransmitStatistics(CRetryCounter *ErrorCtr, VARIANT *ReturnValue);
	void CommClose();
	void Process_TowerData(BYTE inbyte);

	UINT m_MoniTimerID;
	DWORD m_FirmwareThreadID;
	HANDLE m_hFirmwareThread;
	CRetryCounter m_ShortTermRetryCtr;
	CRetryCounter m_LongTermRetryCtr;
	bool IsEcho(BYTE *RecvBuff);
	bool m_bEchoEvent;
	CPollTarget m_Target[MAXEVENTS];
	DWORD m_ReaderThreadID;
	HANDLE m_hReaderThread;
	OVERLAPPED m_WriteOverlap;
	HANDLE m_hReadEvent;
	char m_CommPortStr[5][255];
	CFirmwareData *m_pFirmwareData;
	BYTE m_bSendBuff[MAXBUFF];
	int m_SendLen;
	BYTE m_bRecvBuff[MAXBUFF];
	int m_RecvLen;
	BYTE m_RetryBit;
	HANDLE m_hCom;
	short m_DownloadRetriesLimit;
	short m_ImmidiateRetriesLimit;
	PBRICKOPTIONS m_Brick;
	LINKTYPEOPTIONS m_LinkType;
	COMPORTOPTIONS m_ComPortNo;
	SIZEL m_bmSize;
	HBITMAP m_hBitmap;
	CMessageSink *m_pMessageSink;
};

#endif //__PHANTOMCTRL_H_
