// ProgramData.cpp: implementation of the ProgramData class.
//
//////////////////////////////////////////////////////////////////////
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ProgramData.cpp.
 *
 * The Initial Developer of the Original Code is
 * Fenestra Software LLC.
 * Portions created by the Initial Developer are Copyright (C) 2002 Fenestra Software LLC
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "stdafx.h"
#include "ProgramData.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ProgramData::ProgramData()
{
	m_Image = new BYTE[PGM_MAXLEN];
	memset(m_Image,0,PGM_MAXLEN);
	m_Type = TYPE_TASK;
	m_Length = 0;
	m_Start = 0;
}

ProgramData::~ProgramData()
{
	delete[] m_Image;
}


// Function name	: ProgramData::GetLength
// Description	    : Get the length of the buffer
// Return type		: int 
int ProgramData::GetLength()
{
	return m_Length;
}


// Function name	: *ProgramData::GetData
// Description	    : Get a pointer to the data buffer
// Return type		: BYTE 
BYTE *ProgramData::GetData()
{
	return m_Image;
}


// Function name	: ProgramData::GetStart
// Description	    : Get the Start Address (Not used at this time)
// Return type		: int 
int ProgramData::GetStart()
{
	return m_Start;
}


// Function name	: ProgramData::SetType
// Description	    : Set the type to Task or Sub
// Return type		: void 
// Argument         : RoutineType Type
void ProgramData::SetType(RoutineType Type)
{
	m_Type = Type;
}


// Function name	: ProgramData::GetType
// Description	    : Get the Type
// Return type		: RoutineType 
RoutineType ProgramData::GetType()
{
	return m_Type;
}


// Function name	: ProgramData::GetChecksum
// Description	    : Checksum the buffer
// Return type		: int 
int ProgramData::GetChecksum()
{
	// Compute image checksum
	int cksum = 0;
	int cksumlen = (m_Length < 0x4c00) ? m_Length : 0x4c00;
    for (int i = 0; i < cksumlen; i++){
		cksum += m_Image[i];
	}
	cksum = cksum &0xffff;
	return cksum;
}


// Function name	: ProgramData::SetNumber
// Description	    : Set the Sub or Task Number
// Return type		: void 
// Argument         : int Number
void ProgramData::SetNumber(int Number)
{
	m_Number = Number;
}


// Function name	: ProgramData::GetNumber
// Description	    : Get the Sub or Task Number
// Return type		: int 
int ProgramData::GetNumber()
{
	return m_Number;
}


// Function name	: ProgramData::AddData
// Description	    : Add a new set of bytes to the buffer.
// Return type		: int 
// Argument         : BYTE *InBuff
int ProgramData::AddData(BYTE *InBuff)
{

	int InBuffLen = (*InBuff &0x07);	//Get number of args

	if(InBuffLen > 5) InBuffLen -=6;	//Six and Seven mean 0 and 1

	InBuffLen++;						// Plus one for the command itsself

	while (InBuffLen--){
		m_Image[m_Length++] = *InBuff++;
	}
	return m_Length;
}


// Function name	: ProgramData::DebugDump
// Description	    : Dump the buffer to the debug window.
// Return type		: void 
void ProgramData::DebugDump()
{
	for(int a = 0; a<m_Length; a++){
		AtlTrace("%2.2X ",m_Image[a]);
	}
	AtlTrace("\n");

}

int ProgramData::AddData(RCX_Command &cmd)
{
	return AddData(cmd.GetBody());
}
